%%%%%%%%%%%%%%%%
define cut_dataset_range()
%%%%%%%%%%%%%%%%
%!%+
%\function{cut_dataset_range}
%
%\synopsis{Get get_data_counts-struct clipped to energy range}
%\usage{Struct_Type = cut_dataset_range (hist_index , E_min , E_max);}
%\description
%       This function will return a structre just as get_data_counts
%       but clipped to the energy range from E_min up to E_max (energies in keV). 
%\example
%       isis>id = load_data("data.fits");
%       isis>variable cut_spectrum = cut_dataset_range (id , 0.2 , 2.3);
%\seealso{get_data_counts}
%!%-
{
    variable setid , e_min , e_max;
    switch(_NARGS)
    { case 3: (setid , e_min , e_max) = (); }
    { return help(_function_name()); }

    variable data = _A(get_data_counts(setid));
    variable ind = where(data.bin_lo >= e_min and data.bin_hi < e_max);
    variable cnt_kev_flt = struct_filter(data , ind ; copy);
    return cnt_kev_flt;

}


%%%%%%%%%%%%%%%%
define get_count_rate()
%%%%%%%%%%%%%%%%
%!%+
%\function{get_count_rate}
%
%\synopsis{Get different variants of countrates and counts}
%\usage{Double_Type CR = get_count_rate (hist_index);
%or
%Double_Type (CR , CR_err) = get_count_rate (hist_index ; err);}
%\description
%
%       Use this function to retrive background subtracted 
%       countrates (default), countrates, background rates, or counts
%       and respective uncertainties for spectra that have been loaded.
%                   
%\qualifiers{
%    \qualifier{tex}{additional TeXoutput of CRs, choose this if you need
%         values and uncertainties in LaTeX-format (only printed in terminal)}
%    \qualifier{bkg}{0 (default: background subtracted CR), 1 (count rates),
%         2 (background count rate)}
%    \qualifier{fake}{Set this qualifier if you have a faked spectrum and a 
%          faked background (which has not been deleted!); 
%          Assumption: hist_index(bkg) = hist_index(dset) +1 !!}
%    \qualifier{fake_expo}{set exposure time of fake spectrum, in case 
%               set_data_exposure was not used when faking}
%    \qualifier{bkg_id}{set hist_index(bkg) if background is loaded or faked, if set to 0, no background
%            is subtracted. Will overrule the id-setting made by fake-qualifier}
%    \qualifier{E_range}{Set this qualifier if you want the countrate within
%             a certain energy range. Use E_range = [E_min , E_max].
%             If not given full energy range is used. (Energy in keV!)}
%    \qualifier{counts}{if present function will only return counts instead
%             of countrates}
%    \qualifier{err}{if present function will return CR and
%             respective uncertainty}
%}
%
%\example
%       isis>xray = load_data("data.pha");
%       isis>variable countrate = get_count_rate (1 ;tex, bkg=1 , E_range = [0.2 , 2.3]);
%       isis>variable counts , counts_err;
%       isis>(counts , counts_err) = get_count_rate (1 ;tex, bkg=1 , E_range = [0.2 , 2.3] , counts , err);
%
%\seealso{get_data_counts; TeX_value_pm_error; cut_dataset_range;}
%!%-
{
    variable dset;
    switch(_NARGS)
    { case 1: dset  = (); }
    { help(_function_name()); return; }

    variable rangelist = qualifier("E_range");
    variable bkg = qualifier("bkg", 0);
    variable counts_data , back_file , load_back , counts_back , counts_err , 
    counts_back_err , cr_data , cr_back , expos , cr_sub , cr_sub_err , cr_err , cr_back_err;

    if (qualifier_exists("fake") or qualifier_exists("bkg_id")){
	% Get Background from faked bkg
        if (qualifier_exists("bkg_id")){
            back_file = qualifier("bkg_id");
                                       }
        else{                                
     	    back_file = dset + 1;
            }
        if (back_file == 0){
            counts_back = 0.0;
                           }
        else{                   
            if (qualifier_exists("E_range")){
                 counts_back = sum(cut_dataset_range(back_file , rangelist[0] , rangelist[1]).value);
                                            }
            else {                               
             	counts_back = sum(_A(get_data_counts(back_file)).value);
                 }
            } 
                                                               }
    else{   
	% Load back and get back counts
 	back_file = (get_data_info(dset).bgd_file);
	load_back = load_data(back_file);
    if (qualifier_exists("E_range")){
        counts_back = sum(cut_dataset_range(load_back , rangelist[0] , rangelist[1]).value);
                                    }
    else {
	    counts_back = sum(_A(get_data_counts(load_back)).value);
         }
	% Delete background
	delete_data(load_back);
        }

    % GET BACKSCALES
    variable db = get_data_backscale(dset);
    variable bb = get_back_backscale(dset);
    variable fac = db/bb;
    counts_back = counts_back*fac;

    % GET SOURCE COUNTS
    if (qualifier_exists("E_range")){
        counts_data = sum(cut_dataset_range(dset , rangelist[0] , rangelist[1]).value);                          
                                    }
    else {                               
        counts_data = sum(_A(get_data_counts(dset)).value);
         }
    counts_err = sqrt(counts_data); 
	
    % Get back counts Err
    counts_back_err = sqrt(counts_back);

    % Get background subtracted countrate
    expos = get_data_exposure(dset);

    if (qualifier_exists("fake_expo")){
        expos = qualifier("fake_expo");
                                      }

    if (qualifier_exists("counts")){
        expos = 1.0;
                                   }

    cr_data = counts_data/expos;
    cr_err = counts_err/expos;

    cr_back = counts_back/expos;
    cr_back_err = counts_back_err/expos;

    cr_sub = cr_data - (cr_back);
    cr_sub_err = sqrt((cr_err)^2 - (cr_back_err)^2);
    
    % Check for Qualifier
    variable check_cr;
    
    if (bkg == 0){
    	if (qualifier_exists("tex")){
    	     vmessage(TeX_value_pm_error(cr_sub , cr_sub-cr_sub_err , cr_sub+cr_sub_err));
    	                            }
        if (qualifier_exists("err")){                            
     	    return cr_sub , cr_sub_err;
                                    }
        else{
            return cr_sub;     
            }
                 }
    
    else if (bkg == 1){
	    if (qualifier_exists("tex")){
	        vmessage(TeX_value_pm_error(cr_data , cr_data-cr_err , cr_data+cr_err));
	                                }
        if (qualifier_exists("err")){                            
    	    return cr_data , cr_err;
                                    }
        else{
            return cr_data;
            }
                      }
    else if (bkg == 2){
	    if (qualifier_exists("tex")){
	        vmessage(TeX_value_pm_error(cr_back , cr_back-cr_back_err , cr_back+cr_back_err));
	                                }
        if (qualifier_exists("err")){                            
	        return cr_back , cr_back_err;
                                    }
        else{
            return cr_back;
            }
                      }
    else {
	    vmessage("Wrong Input for Bkg qualifier!");
         }
}
