
define struct_array_2_struct_of_arrays()
%!%+
%\function{struct_array_2_struct_of_arrays}
%\synopsis{Converts a Struct_Type[] to Struct_Type with array fields}
%\usage{Struct_Type struct_array_2_struct_of_arrays(Struct_Type[] SA);}
%\qualifiers{
%\qualifier{depth [0]:}{ If a field is a Struct_Type itself,
%                        the this function calls itself iteratively
%                        'depth' times going down the rabbit hole.}
%\qualifier{dim [0]:}{ If a field is multi-dimensional 'dim' is the
%                       dimension to which the array is appended to.}
%}
%\description
%   This function converts a Struct-Array 'Struct_Type[]' to
%   a single Struct_Type by converting the COMMON fields of
%   all structs into arrays.
%   The function works iteratively in case a field of the given
%   struct array SA itself is a Struct_Type. The function will
%   call itself maximaly 'depth' times.
%   In case a field is multi-dimensional the array will be appended
%   to the 'dim'-th dimension.
%
%   NOTE that this function runs orders faster than 'merge_struct_arrays'
%   while offering the same features as, e.g., reshaping multi-dimensional
%   arrays (the reshaping in this function also works if not every field
%   is multidimensional!).
%
%   The only drawback is that the function fails if an entry is NULL as
%   this case cannot be catched. In that case you may use
%   'merge_struct_arrays'!
%\examples
%
%   %%%%% SIMPLE example
%   variable sa = struct_array( 3, struct{ a=1, b=2 } );
%   sa[0] = struct_combine( struct{c}, sa[0] );
%   variable s = struct_array_2_struct_of_arrays( sa );
%   print(s);
%   {a=Integer_Type[3],
%    b=Integer_Type[3]}
%    
%   %%%%% COMPLEX example
%   variable sa = struct_array( 30000, struct{ a=1,
%                                              b=_reshape([1:120],[1,2,60]),
%                                              c=struct{ d="hello" }
%                                            }
%                             );
%   %%% only merge the parent structure
%   variable s = struct_array_2_struct_of_arrays( sa ; depth=0 );
%   isis>    print(s);
%   %  {a=Integer_Type[30000],
%   %   b=Integer_Type[30000,2,60],
%   %   c=Struct_Type[30000]}
%
%   %%% merge also the first generation     
%   variable s = struct_array_2_struct_of_arrays( sa ; depth=1 );
%   print(s);
%   %  {a=Integer_Type[30000],
%   %   b=Integer_Type[30000,2,60],
%   %   c=Struct_Type with 1 fields}
%   print(s.c);
%   %  {d=String_Type[3]}
%
%   %%% use the second dimension of mudlti-dimensional arrays for merging:
%   variable s = struct_array_2_struct_of_arrays( sa ; dim=1 );
%   print(s);
%   %  {a=Integer_Type[30000],
%   %   b=Integer_Type[1,60000,60],
%   %   c=Struct_Type[30000]}
%
%   %%% Runtime comparision to 'merge_struct_array'
%    tic;  s = struct_array_2_struct_of_arrays( sa );
%    vmessage(" sa2soa took %g seconds to call",toc);
%    print(s);
%    tic;  m = merge_struct_arrays( sa );
%    vmessage(" msa took %g seconds to call",toc);
%    print(m);
%      
%\seealso{merge_struct_array, array_struct_field, array_flatten, unique, intersection}
%!%-
{
  variable _sa;
  switch(_NARGS)
  { case 1: _sa = (); }
  { help(_function_name); }
  
  %%% if 'sa' is no array just return it
  ifnot( typeof (_sa) == Array_Type){
    return array_flatten( _sa[wherenot(_isnull(_sa))] );
  }

  %%% if length(sa) == 1 return sa[0]
  if( length(_sa) == 1 ){
    return _sa[0];
  }

  %%% remove NULL entries
  variable sa = _sa[wherenot(_isnull(_sa))];
  
  %%% QUALIFIERS
  variable depth = qualifier("depth",0);
  variable rdim  = qualifier("dim",0);
  
  %%% get all fieldnames
  variable fnames = array_map( Array_Type, &get_struct_field_names, sa );
  fnames = fnames[unique(fnames)];
  
  %%% find the common fields which are present in all Structs in the array
  variable cfnames = length(fnames)==1 ? fnames[0] : fnames[0][ intersection( __push_array(fnames) ) ];
  
  %%% Do the conversion to a Struct of array_s
  variable s;
  if( _typeof(sa) == Struct_Type ){
    s = @(_typeof(sa))(cfnames);
  }
  else{
    s = @_typeof(sa);
  }
  
  variable fn, fa, fadim, fadimarray;
  foreach fn(cfnames){
    fa = array_struct_field( sa, fn );
    if( _typeof(fa) == Array_Type ){
      fa = fa[wherenot(_isnull(fa))];
      fadim = array_shape(fa[0]);
      if( length(fadim) > 1 ){
	fadimarray = array_flatten( array_map( Array_Type, &array_shape, fa) );
	fadim[rdim] = nint(sum(fadimarray[[rdim:length(fadimarray)-1:length(fadim)]]));
	fa = array_flatten( fa ;; __qualifiers );
	reshape( fa, fadim );
      }
      else{
	fa = array_flatten( fa ;; __qualifiers );
      }
    }
    else if( _typeof(fa) == Struct_Type && depth > 0 ){
      fa = struct_array_2_struct_of_arrays( fa ; depth=depth-1 );
    }
    set_struct_field( s, fn, fa );
  }
  return s;
}
