define KeplerEquation()
%!%+
%\function{KeplerEquation}
%\synopsis{solves Kepler's Equation  E - e sin E  =  M }
%\usage{Double_Type E = KeplerEquation(Double_Type M, e)}
%\description
%    Solve Kepler's Equation by using the result of the method
%    by S. Mikkola (1987) Celestial Mechanics, 40, 329-334
%    as starting value for a Newton-Raphson iteration
%    to extend the applicability of this function to higher eccentricities.
%\qualifiers{
%\qualifier{threshold}{stopping criterion for the Newton Raphson iteration, default = 1e-5}
%}
%!%-
{
  variable M, e;
  switch(_NARGS)
  { case 2: (M, e) = (); }
  { help(_function_name()); return; }
  M = [M];

  variable threshold = qualifier("threshold", 1e-5);
  if(e<0 || e>=1)
  { message("error (%s): eccentricity must be between 0 and 1");
    return;
  }

  % range reduction  =>  -pi < M < pi
  M = M mod (2*PI);
  M[where(M<PI)] += 2*PI;
  M[where(M>PI)] -= 2*PI;

  % bail out for circular orbits
  if(e==0)  return M;

  variable aux = 4*e + 0.5;
  variable alpha = (1-e)/aux;
  variable beta = M/(2*aux);
  aux = sqrt(beta^2 + alpha^3);
  variable z = beta + aux;
  variable i = where(z <= 0.0);
  z[i] = beta[i] - aux[i];
  z = abs(z)^(1./3);
  variable s0 = z - alpha/z;
  variable s1 = s0 - (0.078*s0^5)/(1.+e);
  variable e0 = M + e*(3.*s1 - 4.*s1^3);
  variable se0 = sin(e0);
  variable ce0 = cos(e0);
  variable f = e0 - e*se0 - M;
  variable f1 = 1. - e*ce0;
  variable f2 = e*se0;
  variable f3 = e*ce0;
  variable f4 = -f2;
  variable u1 = -f/f1;
  variable u2 = -f/(f1 + 0.5*f2*u1);
  variable u3 = -f/(f1 + 0.5*f2*u2 + f3*u2*u2/6.);
  variable u4 = -f/(f1 + 0.5*f2*u3 + f3*u3*u3/6. + f4*u3^3/24.);
  variable eccanom = e0 + u4;
  eccanom[where(eccanom >= 2*PI)] -= 2*PI;
  eccanom[where(eccanom < 0)]     += 2*PI;

  % Now get more precise solution using Newton Raphson method
  % for those times when the Kepler equation is not yet solved
  % to better than 1e-10 (modification J. Wilms)
  M[where(M<0.)] += 2.*PI;
  foreach i (where(M != 0 and abs(eccanom - e*sin(eccanom) - M) > 1e-10))
  { if(qualifier_exists("verbose"))
      message("Newton Raphson");
    do
    {
      variable fe = eccanom[i] - e*sin(eccanom[i]) - M[i];  % f  = E - e sin E - M
      variable fs = 1. - e*cos(eccanom[i]);              ;  % f' = 1 - e cos E
      variable oldval = eccanom[i];
      eccanom[i] = oldval - fe/fs;
    } while(abs(oldval-eccanom[i]) > threshold);
    % the following should be coded more intelligently ;-)
    % (similar to range reduction of mx...)
    while(eccanom[i] >= PI)  eccanom[i] -= 2*PI;
    while(eccanom[i] <  PI)  eccanom[i] += 2*PI;
  }

  if(length(eccanom)>1)
    return eccanom;
  else
    return eccanom[0];
}
