define Roche_critical()
%!%+
%\function{Roche_critical}
%\synopsis{calculates the critical potential at L1 and the corresponding radius in z-direction}
%\usage{Struct_Type crit = Roche_critical(Double_Type mq);}
%\description
%  The Roche potential describes the motion of a binary consis-
%  ting of two objects, M1 and M2. The mass ratio mq = M1/M2
%  must be given. From the position of the first Lagrange point
%  'xL1', which is a saddle point in the Roche potential, the
%  value of the critical potential 'critpot' at this point is
%  calculated. The corresponding equipotential surface describes
%  the inner most closed surface, which is also known as 'Roche
%  lobe'. Since the Roche potential does not depend on the
%  binary rotation, the radius 'rcrit' of the star in z-direction
%  (perpendicular to the orbital plane) is often used as the
%  star's radius.
%  
%  All three values are returned by the following structure:
%    xL1     - distance of M1 to L1 in units of the binary
%              displacement
%    critpot - value of the critical potential at L1
%    rcrit   - radius of M1 in z-direction in units of the
%              binary displacement
%\seealso{Roche_potential}
%!%-
{
  variable mq;
  switch(_NARGS)
    { case 1: mq = (); }
    { help(_function_name()); return; }

  % position of L1
  variable xL1 = find_function_maximum(&Roche_potential, 1e-8, 1.-1e-8; qualifiers = struct{ q=mq });
  % value of critical potential at L1
  variable critpot = Roche_potential(xL1; q=mq);
  % critical radius in z-direction
  variable rcrit = find_multiargumentfunction_value(&Roche_potential, critpot, 0.0, 0.0, [1e-8, xL1]; qualifiers = struct{ q=mq });

  return struct { xL1 = xL1, critpot = critpot, rcrit = rcrit };
}
