require("vector");

define geographic2vector()
%!%+
%\function{geographic2vector}
%\synopsis{Calculate a geocentric vector from geographic or geodetic coordinates.}
%\usage{Vector_Type=geographic2vector(lambda,phi,hei;qualifiers)}
%\qualifiers{
%   \qualifier{deg}{if set, then lambda and phi are measured in degrees,
%                     (default: radian).}
%   \qualifier{datum}{a string defining the geodetic datum. The default
%                     is WGS 84, which corresponds to GPS coordinates. Other
%                     geodetic data are IERS 2010, GRS 80, IAU 1976, GRS 67,
%                     IAU 1964, Hayford (1924), Clarke (1866), and Airy (1830),
%                     per the parameters given in the Astronomical Almanac.}
%   \qualifier{geocentric}{The coordinates are not geodetic, but geocentric. 
%                     In this case hei is interpreted as the distance from the
%                     geocenter (otherwise it is the height with respect to
%                     the reference ellipsoid).
%                     Note: it is very rare for coordinates to be geocentric!
%                     GPS is geodetic.}
%}
%\description
%
% For a given set of geodetic coordinates, i.e., geographic longitude (positive
% towards the east!), latitude (positive on the Northern hemisphere), and a
% height  above the reference spheroid (close to height above mean sea level),
% calculate the x,y,z-position in a geocentric coordinate system. Here, the
% xy-plane is the Earth's equator, the xz-plane the ITRS meridian (close to
% Greenwich), and the z-axis points towards the Earth's north pole. The xyz-
% values are in meters.
%
% Note that the longitude and latitude are assumed to be in rad, unless the
% deg qualifier is given. The height is in meters.
%
% The routine returns a Vector_Type. If called with arrays, then an array
% of Vector_Type is returned.
%
%\seealso{dms2deg}
%!%-
{
    variable lam,phi,hei;

    switch(_NARGS)
    { case 3: (lam,phi,hei)=(); }
    { throw UsageError,sprintf("%s: Need longitude, latitude, and height!\n",_function_name());}

    variable la,ph,h;
    if (qualifier_exists("deg")) {
	la=lam*PI/180.;
	ph=phi*PI/180.;
	h=hei;
    } else {
	if (Vector_Type==typeof(lam)) {
	    la=lam[*];
	    ph=phi[*];
	    h=hei[*];
	} else {
	    la=lam;
	    ph=phi;
	    h=hei;
	}
    }

    variable vecarr,i;
    if (qualifier_exists("geocentric")) {
	if (Vector_Type==typeof(la)) {
	    vecarr=Vector_Type[length(la)];
	    _for i (0,length(la),1) {
		vecarr[i]=vector_astro(h[i],la[i],ph[i];astro);
	    }
	    return vecarr;
	} 
	return vector_astro(h,la,ph;astro);
    }

    %
    % geoid parameters for different datums
    % (see Astronomical Almanac, section K)
    %
    variable a,f;
    variable datum=qualifier("datum","WGS 84");
    switch(datum) 
    {case "WGS 84"          : a=6378137.0  ; f=1./298.257223563; }
    {case "IERS 2010"       : a=6378136.6  ; f=1./298.25642;     }
    {case "GRS 80"          : a=6378137.0  ; f=1./298.257222;    }
    {case "IAU 1976"        : a=6378140.0  ; f=1./298.257;       }
    {case "GRS 67"          : a=6378160.0  ; f=1./298.247167;    }
    {case "IAU 1964"        : a=6378160.0  ; f=1./298.25;        }
    {case "Hayford (1924)"  : a=6378388.0  ; f=1./297.;          }
    {case "Clarke (1866)"   : a=6378206.4  ; f=1./294.978698;    }
    {case "Airy (1830)"     : a=6377563.396; f=1./299.324964;    }
    {throw UsageError,sprintf("%s: Geographic datum %s not known\n",_function_name(), datum);}

    %
    % convert to x,y,z
    %
    variable sp,cp,sl,cl;
    (sp,cp)=sincos(ph);
    (sl,cl)=sincos(la);

    variable C=1./sqrt(cp*cp+(1.-f)*(1.-f)*sp*sp);
    variable S=C*(1.-f)*(1.-f);

    if (Vector_Type==typeof(la)) {
	vecarr=Vector_Type[length(la)];
	_for i(0,length(la)-1,1) {
	    vecarr[i]= vector((a*C[i]+h[i])*cp[i]*cl[i], (a*C[i]+h[i])*cp[i]*sl[i], (a*S[i]+h[i])*sp[i]);
	}
	return vecarr;
    }
    return vector((a*C+h)*cp*cl, (a*C+h)*cp*sl, (a*S+h)*sp);
}
