%
% Functions to handle JPL old-style (binary, non-SPK)
% ephemerides
%
% Joern Wilms, Sept. 2019 - Dec. 2019
%

require("vector");

define jpl_bineph2fits(fitsfile,ephfile) 
%!%+
%\function{jpl_bineph2fits}
%\synopsis{convert a binary JPL ephemeris to FITS}
%\usage{jpl_bineph2fits(ephfile,fitsfile);}
%\description
% This function reads the JPL binary ephemeris in ephfile
% and writes it to the fitsfile.
%
% The JPL ephemeris must be in the "old-style" format.
% SPK files are NOT supported.
%
% The function has to be run on a machine with the same byte
% sex as the ephemeris (JPL typically distributes ephemerides
% for all common types of byte sex). It does not do a lot of
% checking, however, it has been used to successfully
% convert ALL available JPL old-style ephemerides into FITS.
%
% Which also means you should probably ask Joern Wilms for
% the location of the relevant FITS file rather than using this
% function...
%
%\seealso{jpl_eph,jpl_eph_vec}
%!%-
{
    %
    % read data into a large structure
    %
    variable ifile=fopen(ephfile,"rb");

    variable sizeof_double=8;
    variable sizeof_int32=4;

    variable tit;
    ()=fread_bytes(&tit,84,ifile); % read the title

    ()=fseek(ifile,2652, SEEK_SET);  % buffer starts 2652 bytes past the start

    % header constants 
    variable buf;
    % validity interval
    ()=fread_bytes(&buf,sizeof_double,ifile);
    variable ephem_start=unpack("d",buf);
    ()=fread_bytes(&buf,sizeof_double,ifile);
    variable ephem_end=unpack("d",buf);
    ()=fread_bytes(&buf,sizeof_double,ifile);
    variable ephem_step=unpack("d",buf);

    % number of constants
    ()=fread_bytes(&buf,sizeof_int32,ifile);
    variable ncon=unpack("k",buf);
    
    if (ncon>65535) {
	print("byte sex of binary file is wrong!");
	exit;
    }

    % AU
    ()=fread_bytes(&buf,sizeof_double,ifile);
    variable au=unpack("d",buf);
    % earth moon mass ratio
    ()=fread_bytes(&buf,sizeof_double,ifile);
    variable emrat=unpack("d",buf);

    % value from jpleph, I don't know where it is in the header
    variable pvsun_t=-1e80;
    variable ipt=Integer_Type[40,3];
    variable i;
    _for i(0,39,1) {
	()=fread_bytes(&buf,sizeof_int32,ifile);
	ipt[i/3,i mod 3]=unpack("k",buf);
    }

    variable ephemeris_version=atoi(substr(tit,27,strlen(tit)));

    % see jpleph.cpp
    % I'm not sure why the reading isn't done in a different way
    ipt[12,0]=ipt[12,1];
    ipt[12,1]=ipt[12,2];
    ipt[12,2]=ipt[13,0];

    % decision on whether the ephemeris includes TT-TDB data
    if (ephemeris_version >=430 && ncon!=400 ) {
	if (ncon>400) {
	    ()=fseek(ifile, (ncon-400)*6,SEEK_CUR);
	}
	_for i(0,6) {
	    ()=fread_bytes(&buf,sizeof_int32,ifile);
	    ipt[13+i/3,i mod 3]=unpack("k",buf);
	}
    } else {
	ipt[13,0]=-1;
    }

    % check if there's garbage in these values
    if (ipt[13,0] != ipt[12,0]+ipt[12,1]*ipt[12,2]*3 ||
    ipt[14,0] != ipt[13,0]+ipt[13,1]*ipt[13,2]*3 ) {
	()=printf("Info: No valid TT-TDB pointer\n");
	ipt[[13:14],[0:2]]=0;
    }

    variable kernel_size=4;
    _for i(0,14,1) {
	variable dim=3; % most parameters have xyz values
	if (i==11) {
	    dim=2; % nutations: 2 values
	}
	if (i==14) {
	    dim=1; % tdt-tt
	}
	kernel_size+=2*ipt[i,1]*ipt[i,2]*dim;
    }

    variable recsize=kernel_size*4;
    variable ncoeff=kernel_size/2;

    % read constant numbers and values
    ()=fseek(ifile,recsize,SEEK_SET);
    variable val=Double_Type[ncon];
    variable tmp=fread(&val,Double_Type,ncon,ifile);

    variable bufoffset=84*3+400*6+5*sizeof_double+41*sizeof_int32;
    variable nam=String_Type[ncon];
    ()=fseek(ifile,84*3,SEEK_SET);
    _for i(0,ncon-1,1) {
	if (i==400) {
	    ()=fseek(ifile,bufoffset,SEEK_SET);
	}
	()=fread_bytes(&buf,6,ifile);
	nam[i]=strtrim(buf);
    }

    % 
    variable idnam=[
    "Mercury", 
    "Venus",   
    "Earth",   
    "Mars",    
    "Jupiter", 
    "Saturn",  
    "Uranus",  
    "Neptune", 
    "Pluto",   
    "Moon",    
    "Sun",     
    "Nutations",               
    "Lunar libration (Euler) angles",
    "Lunar angular velocity",
    "TT-TDB"                   
    ];

    variable origin="J. Wilms, joern.wilms@sternwarte.uni-erlangen.de";
    variable creator="bineph2fits.sl, version 1.0";

    variable de1_keys=struct {
	EXTNAME="DE1",
	ORIGIN=origin,
	CREATOR=creator
    };

    variable de1_data=struct {
	Cname=nam[*],
	Cvalue=val[*]
    };

    variable de2_keys=struct {
	EXTNAME="DE2",
	ORIGIN=origin,
	CREATOR=creator
    };

    variable onum=[0:length(idnam)-1];
    variable de2_data=struct {
        Object=idnam[*],
        Pointer=ipt[onum,0],
        NumCoeff=ipt[onum,1],
        NumSubIntv=ipt[onum,2]
    };

    % write the first two extensions
    variable fptr=fits_open_file(fitsfile,"c");
    fits_write_binary_table(fptr,"DE1",de1_data,de1_keys);
    fits_write_date(fptr);
    fits_write_chksum(fptr);
    fits_write_binary_table(fptr,"DE2",de2_data,de2_keys);
    fits_write_date(fptr);
    fits_write_chksum(fptr);

    % now read ephemeris records
    % we need to do this piece by piece since some ephemerides
    % can be too long for isis to handle in one sitting

    variable nrec=int((ephem_end-ephem_start)/ephem_step);
    
    fits_create_binary_table(fptr,"DE3",nrec,["ChebCoeffs"],[sprintf("%iD",ncoeff)],[""]);
    fits_update_key(fptr,"EXTNAME","DE3");
    fits_update_key(fptr,"ORIGIN",origin);
    fits_update_key(fptr,"CREATOR",creator);
    fits_update_key(fptr,"TSTART",ephem_start);
    fits_update_key(fptr,"TSTOP",ephem_end);
    fits_update_key(fptr,"TIMEDEL",ephem_step);
    fits_update_key(fptr,"TIMESYS","TT");
    fits_update_key(fptr,"TIMEUNIT","d");
    fits_update_key(fptr,"JDREF",0.);
    fits_update_key(fptr,"TDIM1",sprintf("(%i)",ncoeff));

    variable nr;
    _for nr(0,nrec-1,1) {
        % we could probably omit this
        ()=fseek(ifile,(nr+2)*recsize,SEEK_SET);
        buf=Double_Type[ncoeff];
        ()=fread(&buf,Double_Type,ncoeff,ifile);
        variable coeffs=@buf;
        ()=_fits_write_col(fptr,1,nr+1,1,coeffs[*]);
    }
    fits_write_date(fptr);
    fits_write_chksum(fptr);

    fits_close_file(fptr);
}

private variable jplname=Assoc_Type[Int_Type];

private define jpl_name2jplid(name) {
    %
    % convert the name of an object to the
    % object's number in the old-style JPL ephemeris
    %
    
    %  1 - Mercury
    %  2 - Venus 
    %  3 - Earth (geocenter)
    %  4 - Mars (system barycenter)
    %  5 - Jupiter (system barycenter)
    %  6 - Saturn (system barycenter)
    %  7 - Uranus (system barycenter)
    %  8 - Neptune (system barycenter)
    %  9 - Pluto (system barycenter)
    % 10 - Moon
    % 11 - Sun
    % 12 - Solar System Barycenter
    % 13 - Earth-Moon Barycenter
    % 14 - 1980 IAU nutation angles
    % 15 - Lunar libration (Euler) angles
    % 16 - Lunar angular velocity
    % 17 - TT-TDB (at geocenter)

    if (assoc_key_exists(jplname,"initialized")==0) {
	jplname["MERCURY"]=  1;
	jplname["VENUS"]=  2;
	jplname["EARTH"]=  3;
	jplname["MARS"]=  4;
	jplname["JUPITER"]=  5;
	jplname["SATURN"]=  6;
	jplname["URANUS"]=  7;
	jplname["NEPTUNE"]=  8;
	jplname["PLUTO"]=  9;
	jplname["MOON"]= 10;
	jplname["SUN"]= 11;
	jplname["SOLAR SYSTEM BARYCENTER"]= 12;
	jplname["SSB"]= 12;
	jplname["EARTH-MOON BARYCENTER"]= 13;
	jplname["EARTH-MOON"]= 13;
	jplname["EMB"]= 13;
	jplname["1980 IAU NUTATION ANGLES"]= 14;
	jplname["IAU1980"]= 14;
	jplname["LUNAR LIBRATION (EULER) ANGLES"]= 15;
	jplname["LUNAR LIBRATION"]= 15;
	jplname["LUNAR ANGULAR VELOCITY"]= 16;
	jplname["TT-TDB"]=17;
	jplname["initialized"]=-1;
    }

    if (assoc_key_exists(jplname,strup(name))) {
	return jplname[strup(name)];
    }

    throw UsageError(sprintf("Object name does not exist: %s\n",name));
}

define jpl_initeph() 
%!%+
%\function{jpl_initeph}
%\synopsis{initialize a JPL ephemeris}
%\usage{Struct_Type results = jpl_initeph(file);}
%\description
% This function initializes a JPL ephemeris for later use with
% function jpl_eph.
%
% For an empty argument, the function defaults to DE430 as distributed
% with HEASOFT. This is probably what you want (and since jpl_eph
% initializes to that ephemeris, this means that you probably do NOT
% want to call this function in the first place).
%
% The ephemeris file must be in FITS format, using the HEASOFT
% convention (see function jpl_bineph2fits if you need to generate
% such a file from one of the older JPL ephemeris distributions).
%
% The function first searches for the ephemeris file as given,
% and then searches for the file by prepending ${LHEA_DATA}/
% or ${LHEASOFT}/refdata/. 
%
%\qualifiers{
%  \qualifier{nocache}{Do not cache the ephemeris data in memory,
%                        i.e., keep the FITS file open and seek on it.
%                        This is faster if calculating just a few positions
%                        (e.g., for a few weeks), otherwise, reading the
%                        whole ephemeris first [the default] is faster,
%                        but a memory hog, i.e., it won't work for DE431.}
%}
%\seealso{jpl_eph,jpl_bineph2fits}
%
%!%-
{
    variable ephfile;
    switch(_NARGS)
    {case 0: ephfile="JPLEPH.430";}
    {case 1: ephfile=();}
    { throw UsageError, sprintf("%s: Invalid number of arguments\n",_function_name()); }

    % search for the file
    variable fitsfile=ephfile;
    if (stat_file(fitsfile)==NULL) {
        if (getenv("LHEA_DATA")!=NULL) {
	    fitsfile=getenv("LHEA_DATA")+"/"+ephfile;
	} else {
	    if (getenv("LHEASOFT")!=NULL) {
		fitsfile=getenv("LHEASOFT")+"/refdata/"+ephfile;
	    } else {
		throw UsageError,sprintf("%s: eph not given and LHEA_DATA or LHEASOFT not set!",_function_name());
	    }
	}
	if (stat_file(fitsfile)==NULL) {
	    throw UsageError,sprintf("%s: cannot find ephfile %s",_function_name,ephfile);
	}
     }
    variable fptr=fits_open_file(fitsfile,"r");

    variable ret=_fits_movnam_hdu(fptr,_FITS_BINARY_TBL,"DE1",0);
    variable hdr=fits_read_header(fptr);
    variable const=fits_read_table(fptr);

    variable nn=where(const.cname=="AU");
    if (length(nn)==0) {
	throw RunTimeError,sprintf("%s: Ephemeris file %s does not contain a value for AU.\n",_function_name,fitsfile);
    }
    variable au=const.cvalue[nn[0]];

    nn=where(const.cname=="EMRAT");
    if (length(nn)==0) {
	throw RunTimeError,sprintf("%s: Ephemeris file %s does not contain a value for the Earth-Moon mass ratio.\n",_function_name,fitsfile);
    }
    variable emrat=const.cvalue[nn[0]];

    nn=where(const.cname=="CLIGHT");
    if (length(nn)==0) {
	throw RunTimeError,sprintf("%s: Ephemeris file %s does not contain a value for the speed of light.\n",_function_name,fitsfile);
    }
    variable clight=const.cvalue[nn[0]];

    nn=where(const.cname=="GMS");
    if (length(nn)==0) {
	throw RunTimeError,sprintf("%s: Ephemeris file %s does not contain a value for GM for the Sun.\n",_function_name,fitsfile);
    }
    variable gms=const.cvalue[nn[0]];
    
    ()=_fits_movnam_hdu(fptr,_FITS_BINARY_TBL,"DE2",0);
    variable pars=fits_read_table(fptr);

    ()=_fits_movnam_hdu(fptr,_FITS_BINARY_TBL,"DE3",0);
    variable jdstart=fits_read_key(fptr,"TSTART");
    variable jdstop=fits_read_key(fptr,"TSTOP");
    variable dt=fits_read_key(fptr,"TIMEDEL");

    if (qualifier_exists("nocache")) {
	return struct {
	    pars=pars,
	    fptr=fptr,
	    jdstart=jdstart,
	    jdstop=jdstop,
	    dt=dt,
	    au=au,
	    emrat=emrat,
	    clight=clight,
	    gms=gms,
	    cache=0
	};
    }

    variable coeffs=fits_read_table(fptr);
    fits_close_file(fptr);

    return struct {
	pars=pars,
	coeffs=coeffs,
	jdstart=jdstart,
	jdstop=jdstop,
	dt=dt,
	au=au,
	emrat=emrat,
	clight=clight,
	gms=gms,
	cache=1
    };
}


% cache for chebyshev polynomial
private variable jpl_ephcache=struct {
    tc=-2., 
    cheby=Double_Type[30],    % this is larger than in all published JPL eph 
    dercheby=Double_Type[30], % that used the old JPL format
    ncf=-1,
    chebcoeffs=[0], % cache for ephemeris chebyshev coefficients
    recno=-1        % (only if reading from a file)
};

private define jplinter(jd,jplindex,eph) {
    %
    % perform the interpolation in the JPL ephemeris
    % This is a private routine that should never be called
    % from anywhere else than jpl_eph
    %
    % Because of the way we're reading from the ephemeris and
    % because of the caching of the Chebyshev polynomials, this
    % routine is NOT safe to be used in parallel_map and other
    % parallel routines!!
    %
    
    % ephemeris record to use
    variable fltrecno=(jd[0]-eph.jdstart + jd[1])/eph.dt;
    
    variable recno=typecast(fltrecno,Long_Type);
    variable trel=fltrecno-recno;
    % boundary case (plus possible roundoff correction)
    if (trel<=0.) {
	trel=1.;
	recno=recno-1;
    }
    
    %
    % parameters for this component
    % ncf: number of coefficients per component
    % na : number of sub-intervals in the full interval

    if (length(eph.pars.numcoeff)<= jplindex-1) {
        throw UsageError(sprintf("Ephemeris has no information for jplindex=%i\n"),jplindex);
    }
    
    variable ncf=eph.pars.numcoeff[jplindex-1];
    variable na =eph.pars.numsubintv[jplindex-1];

    if (ncf==0) {
        throw UsageError(sprintf("Ephemeris has no information for jplindex=%i\n"),jplindex);
    }
    
    % number of coefficients for the jplindex
    variable ncm=3;
    % nutations
    if (jplindex==12) {
	ncm=2;
    }
    % TT-TDB
    if (jplindex==15) {
	ncm=1;
    }

    variable tmp=na*trel;
    variable ell=int(tmp);
    variable tc=2.0*(tmp-typecast(tmp,Long_Type))-1.0;
    
    if (int(tmp) >= na) {
	ell-=1;
	tc+=2.;
    }
    
    % here we should have -1<=tc <=+1

    %
    % evaluate the chebyshev polynomials
    % (we reuse earlier values from our cache if applicable)
    %
    variable i;
    variable t2=2*tc;
    variable ncfstart=2;

    if (tc==jpl_ephcache.tc) {
	ncfstart=jpl_ephcache.ncf+1;
    } else {
	jpl_ephcache.tc=tc;
	jpl_ephcache.cheby[0]=1.;
	jpl_ephcache.cheby[1]=tc;
	jpl_ephcache.dercheby[0]=0.0;
	jpl_ephcache.dercheby[1]=1.0;
    }
    
    % calculate the missing Chebyshev coefficients if necessary
    if (ncfstart<ncf) {
	% note: dercheby[ncf] is not needed, but this makes
	%  the overall logic of the code simpler
	_for i(ncfstart,ncf,1) {
	    jpl_ephcache.cheby[i]=t2*jpl_ephcache.cheby[i-1] - jpl_ephcache.cheby[i-2];
	    jpl_ephcache.dercheby[i]=t2*jpl_ephcache.dercheby[i-1]+2.0*jpl_ephcache.cheby[i-1]-jpl_ephcache.dercheby[i-2];
	}
	jpl_ephcache.ncf=ncf;
    }
    
    % apply the chebyshev polynomial
    variable j;
    variable pos=Double_Type[ncm];
    variable vel=Double_Type[ncm];
    variable vfac=2.*na/(eph.dt*86400.);

    %
    % readable version of the code below
    %
    % % running the loop backwards ensures that cndx goes through
    % % chebcoeffs in one direction only (better for the cache in
    % % compiled languages)
    % _for i(ncm-1,0,-1) {
    % 	% NB: -1 is there because slang and C arrays are 0 based, not 1 based
    % 	variable cndx=eph.pars.pointer[jplindex-1]+ncf*(i+ell*ncm)-1;
	
    % 	% this is faster than 2 loops, even though the vel loop would
    % 	% only have to go to 1 and not to 0 (since dercheby[0]=0)
    % 	_for j(ncf-1,0,-1) {
    % 	    variable jj=cndx+j;
    % 	    pos[i]+=jpl_ephcache.cheby[j]*eph.coeffs.chebcoeffs[recno,jj];
    % 	    vel[i]+=jpl_ephcache.dercheby[j]*eph.coeffs.chebcoeffs[recno,jj];
    % 	}
    % 	vel[i]*=vfac;
    % }

    % The complicated handling of the cache here is because
    % copying ephchebcoeffs over to the local variable chebcoeffs
    % speeds up the code by 10%
    variable chebcoeffs;
    if (eph.cache) {
	chebcoeffs=eph.coeffs.chebcoeffs[recno,*];
    } else {
	if (jpl_ephcache.recno!=recno) {
	    jpl_ephcache.chebcoeffs=fits_read_cell(eph.fptr,1,recno+1);
	    jpl_ephcache.recno=recno;
	}
	chebcoeffs=jpl_ephcache.chebcoeffs[*];
    }

    % NB: -1 for 0 based arrays, 0 for 1-based arrays
    variable cndx=eph.pars.pointer[jplindex-1]+ncf*((ell+1)*ncm)-1;
    _for i(ncm-1,0,-1) {
	_for j(ncf-1,0,-1) {
	    cndx--;
	    pos[i]+=jpl_ephcache.cheby[j]*chebcoeffs[cndx];
	    vel[i]+=jpl_ephcache.dercheby[j]*chebcoeffs[cndx];
	}
	vel[i]*=vfac;
    }
    
    return [pos,vel];
    
    %
    % acceleration (if necessary)
    % variable der2cheby=Double_Type[ncf];
    % der2cheby[0]=0.;
    % der2cheby[1]=0.;
    % _for i(2,ncf-1,1) {
    % 	der2cheby[i]=4*eph.dercheby[i-1]+t2*der2cheby[i-1]-der2cheby[i-2];
    % }

    % variable acc=Double_Type[ncm];
    % _for i(0,ncm-1,1) {
    % 	% NB: -1 is there because slang and C arrays are 0 based, not 1 based
    % 	cndx=eph.pars.pointer[jplindex-1]+ncf*(i+ell*ncm)-1;
    
    % 	_for j(ncf-1,0,-1) {
    % 	    jj=cndx+j;
    % 	    acc[i]+=der2cheby[j]*eph.coeffs.chebcoeffs[recno,jj];
    % 	}
    % 	acc[i]*=vfac*vfac;
    % }
}

% save jpl ephemeris (used if eph argument of jpl_eph is not given)
private variable jpl_ephsave;

define jpl_eph ()
%!%+
%\function{jpl_eph}
%\synopsis{calculate planet positions using the JPL ephemeris}
%\usage{Struct_Type results = jpl_eph(jd,ta,ce);}
%\description
%    This function returns the ephemeris  of target body ta
%    as seen from center body ce for the time jd (the JD in the
%    TDB system). 
%
%    NOTE: To calculate apparent planet positions for observing,
%    use the higher level function planetpos.
%
%    NOTE2: If you want a routine that is compatible with
%    function vsop87, uses TT and is vector safe, use  jpl_eph_vec
%
%    jd is either a Double or a 2-element array (to avoid roundoff
%    errors the array can include, e.g., the integer and fractional
%    parts of the JD). It is given in in TDB. If you need to calculate
%    positions for a time TT (=TAI+32.184s), convert with function
%    tt2tdb.
%    Note, however, that for all practical applications except for ms
%    pulsar timing, you can assume that TT=TDB. 
%
%    The target and center bodies ta and ce are identified as outlined
%    below.
%
%    By default, the routine initializes to DE430 as distributed with HEASOFT
%    (and found either in  $LHEA_DATA/JPLEPH.430 or in
%    $LHEASOFT/refdata/JPLEPH.430).
%    The ephemeris in HEASOFT is only a subset of DE430, valid for about 1950--2050.
%    For these ephemerides, the precision for the decades around AD2000 is on
%    the order of meters to tens of meters, and milliarcseconds (see below). 
%    Joern provides full files for DE430, valid for the time between
%    AD1550 and AD2650, and for DE431, which covers years -13200 to +17191
%    (with lower precision for the moon).
%
%    Use the ephemeris qualifier if you want to use another ephemeris.
%
%    For planets, the function returns an array containing the xyz-position,
%    and the velocity components in the ICRF. For angle variables
%    (only contained in a few of the JPL ephemerides), the array elements are
%    the angles and their  derivatives, and for the difference TT-TDB, the time
%    difference and its derivative. Units are km and km/s, rad and deg/day [!],
%    or s and s/day [!]. 
%
%    Target bodies or angles are either identified by a number as listed
%    below or by the indicated case-insensitive string. Note that not all
%    ephemeris files contain all of the possibilities!
%
%     1 - Mercury
%     2 - Venus 
%     3 - Earth
%     4 - Mars
%     5 - Jupiter
%     6 - Saturn
%     7 - Uranus
%     8 - Neptune
%     9 - Pluto
%    10 - Moon
%    11 - Sun
%    12 - Solar System Barycenter
%    13 - Earth-Moon Barycenter
%    14 - 1980 IAU nutation angles
%    15 - Lunar libration (Euler) angles
%    16 - Lunar angular velocity
%    17 - TT-TDB
%
%    where 3 is the Earth's geocenter. For planets with moons, the 
%    position returned is with respect to the barycenter of the respective
%    planetary system.
% 
%    TT-TDB is measured at the geocenter.
%
%    The following abbreviations are also allowed:
%    12: SSB
%    13: Earth-Moon or EMV
%    14: IAU1980
%    15: Lunar Libration
%
% For targets 14-17 the center is ignored
% 
% See Folkner et al., 2014, IPN Progress Report 42-196 for a description
% of the precision that is reached with DE430/DE431. The figures in
% this publication indicate that for the timeframe from about 1970 onwards,
% where precise astrometry and space probe data exist, a precision
% on the order of several 10s of meters is reached (and apparent
% angular positions are precise to milli-arcseconds).
%
% This function is NOT array safe, and it CANNOT be used in
% a parallel_map or other parallelized functions. This is
% because global caches are used that we cannot avoid if we
% want decent performance for single calls. Sorry
%
% \qualifiers{
%    \qualifier{ephemeris}{structure defining the ephemeris, initialized with
%                            jpl_initeph (see there).}
% }
% 
%\seealso{jpl_eph_vec,jpl_initeph, planetpos}
%!%-
{
    variable jdin,ta,ce;

    switch(_NARGS)
    { case 2: (jdin,ta)=(); }
    { case 3: (jdin,ta,ce)=();  }
    { throw UsageError, sprintf("%s: Invalid number of arguments\n",_function_name()); }

    variable eph;
    if (qualifier_exists("ephemeris")) {
	eph=qualifier("ephemeris");
    } else {
	% ephemeris not given: use the cached emphemeris or open an
	% new one
	% (NB: jpl_initeph with no argument defaults to DE430. It
	% also does all of the error handling
	% and also search in LHEA_DATA and LHEASOFT)
	ifnot (__is_initialized(&jpl_ephsave)) {
	    jpl_ephsave=jpl_initeph();
	}
	eph=jpl_ephsave;
    }

    variable target=ta;
    
    if (typeof(target)==String_Type) {
	target=jpl_name2jplid(target);
    }

    if (_NARGS==2) {
	if (target<14 || target>17) {
	    { throw UsageError, sprintf("%s: Must give center\n",_function_name()); }
	}
	ce=0; % dummy value
    }

    variable center=ce;
    if (typeof(center)==String_Type) {
	center=jpl_name2jplid(center);
    }

    variable jd=@jdin;
    if (typeof(jd)==Double_Type) {
	jd=[jd,0.];
    } else {
	if (typeof(jd)==Array_Type && length(jd)!=2) {
	    throw UsageError,sprintf("%s: jd must be double scalar or double array with 2 entries",_function_name());
	}
    }

    % check validity
    variable jdlow=jd[0]+jd[1];
    if (jdlow<eph.jdstart or jdlow>eph.jdstop) {
	throw UsageError,sprintf("%s: Time %15f is outside validity interval (JD %15f to %15f)\n",_function_name(),jdlow,eph.jdstart,eph.jdstop);
    }
    
    
    if (target<=0 || target > 17) {
	throw UsageError,"Target must be between 1 and 17";
    }

    if (center<=0) {
	if (target<14) {
	    throw UsageError,"Must give center info!";
	}
    }
    if (center>13) {
	if (target<14) {
	    throw UsageError,"Center value does not make sense";
	}
    }

    if (target<=13 && center>=14) {
	throw UsageError,"Center not possible for angles";
    }

    if (center<=13) {
	if (center==target) {
	    return [0.,0.,0.,0.,0.,0.];
	}
    }
    
    %
    % Map object index to index in JPL file
    %
    % (so lots of strange bureaucracy)
    %
    variable jplindex;
    variable pv;
    variable pv2;
    variable pvb;
    variable pvm;

    % ignore center for the angles
    if (14<=target<=17) {
	center=0;
    }
    
    if (target==14) {
	% nutation
	jplindex=12;

	if (eph.pars.pointer[jplindex-1]==0 || eph.pars.numcoeff[jplindex-1]*eph.pars.numsubintv[jplindex-1]==0) {
	    % no nutation values present
	    return -1;
	}
	pv=jplinter(jd,jplindex,eph);
	pv[2]*=86400.; % rad/s -> rad/d
	pv[3]*=86400.; % rad/s -> rad/d
	return pv;
    }

    if (target==15 || target==16) {
	% Lunar libration (Euler) angles
	if (target==15) {
	    jplindex=13;
	} else {
	    jplindex=14;
	}

	if (eph.pars.pointer[jplindex-1]==0 || eph.pars.numcoeff[jplindex-1]*eph.pars.numsubintv[jplindex-1]==0) {
	    % no lunar euler angle values present
	    return -1;
	}
	pv=jplinter(jd,jplindex,eph);
	pv[3]*=86400.; % rad/s -> rad/d
	pv[4]*=86400.; % rad/s -> rad/d
	pv[5]*=86400.; % rad/s -> rad/d
	return pv;
    }

    if (target==17) {
	% TT-TDB
	jplindex=15;

	if (eph.pars.pointer[jplindex-1]==0 || eph.pars.numcoeff[jplindex-1]*eph.pars.numsubintv[jplindex-1]==0) {
	    % no TT-TDB value present
	    return -1;
	}
	pv=jplinter(jd,jplindex,eph);
	pv[1]*=86400.; % s/s -> s/d
	return pv;
    }
    
    %
    % Earth/moon cases
    %
    % target=moon, center=geocenter
    if (target==10 && center==3) {
	jplindex=10; % moon wrt. earth
	return jplinter(jd,jplindex,eph);
    }

    % target=earth, center=moon
    if (target==3 && center==10) {
	jplindex=10; % moon wrt. earth
	return -jplinter(jd,jplindex,eph);
    }

    % earth or moon is involved
    if (target==3 || center==3 || target==10 || center==10) {
	variable facte=1./(1.+eph.emrat);
	variable factm=facte-1.0;

	pvm=jplinter(jd,10,eph); % moon
	pvb=jplinter(jd,3,eph); % E+M Barycenter

	if (target==3 || target==10) {
	    
	    if (center<12) {
		% any of the other planets
		% (note e/m bary has already been taken care of)
		pv2=jplinter(jd,center,eph);
	    } else {
		% EM barycenter
		if (center==13) {
		    pv2=jplinter(jd,3,eph);
		} else {
		    pv2=[0.,0.,0.,0.,0.,0.];
		}
	    }
	    if (target==3) {
		% earth
		pv=pvb-facte*pvm-pv2;
	    } else {
		% moon
		pv=pvb-factm*pvm-pv2;
	    }
	    return pv;
	}

	% target is not earth or moon
	if (target<12) {
	    pv=jplinter(jd,target,eph);
	} else {
	    if (target==13) {
		pv=jplinter(jd, 3,eph);
	    } else {
		pv=[0.,0.,0.,0.,0.,0.];
	    }
	}

	if (center==3) {
	    pv=pv-(pvb-facte*pvm);
	} else {
	    pv=pv-(pvb-factm*pvm);
	}
	return pv;
    }

    % cases not involving Earth or Moon
    % (but might include the EM barycenter)
    variable pv1;
    if (target<12) {
	pv1=jplinter(jd,target,eph);
    } else {
	if (target==13) {
	    pv1=jplinter(jd,3,eph);
	} else {
	    pv1=[0.,0.,0.,0.,0.,0.];
	}
    }

    if (center<12) {
	pv2=jplinter(jd,center,eph);
    } else {
	if (center==13) {
	    pv2=jplinter(jd,3,eph);
	} else {
	    pv2=[0.,0.,0.,0.,0.,0.];
	}
    }
    return pv1-pv2;
}

% forward declaration (necessary for recursive call later!)
define jpl_eph_vec();

define jpl_eph_vec()
%!%+
%\function{jpl_eph_vec}
%\synopsis{high precision planetary positions from the JPL ephemerides, vsop87 compatible}
%\usage{Struct_Type jpl_eph_vec(JD,planet;qualifiers)}
%\description
% This routine provides an interface to jpl_eph which allows to
% calculate heliocentric ephemerides from the JPL ephemerides.
% It is compatible with a subset of the functions that are also
% available from function vsop87.
%
% Use planetpos to calculate apparent planetary positions.
%
% The routine returns high precision planetary positions for
% all planets for a given (array) of times (in TT).
%
% Depending on the qualifiers the routine can return rectangular
% coordinates (X,Y,Z) or ecliptical coordinates
% (longitude, latitude, distance) in a heliocentric coordinate system
% for epoch J2000 (the ICRF).
%
% Contrary to vsop87, orbital elements cannot be calculated. 
%
% By default the function returns rectangular heliocentric
% coordinates in a structure with elements x, y, z, vx, vy, vz.
% Here x, y, z are in an ecliptical(!) coordinate system for
% the epoch J2000 (but see the equatorial qualifier), where the x-axis
% is in the direction of the vernal equinox, the z-axis points
% to the ecliptical North pole, and the y-axis forms
% a right handed coordinate system with the x- and z-axes.
% The default units are AU and AU/day (but see the mks and cgs qualifiers).
%
% For heliocentric coordinates the planet can have the values
% "Mercury", "Venus", "Earth", "Mars", "Jupiter", "Saturn",
% "Uranus", "Neptune", and "Pluto" (the latter is not included
% in vsop87!). The value "Earth-Moon" will return the position
% of the Earth-Moon barycenter.
%
% The default unit of time is the Julian Date in TT, to be
% compatible with vsop87. Use the qualifier tdb to avoid the
% conversion TT->TDB, which for pretty much all practical
% applications except for ms pulsar timing results in no
% appreciable difference in the results but is slightly faster.
%
% Use the mjd qualifier to use MJD rather than JD as input.
%
% The default function uses DE430 from HEASOFT, which is valid
% for the time between 1950 and 2050. Joern can provide ephemeris
% files for the full DE430, valid AD1550 and AD2650, and
% also for DE431, which covers years -13200 to +17191
% (with lower precision for the moon). See the ephemeris qualifier.
%
% When using JPL ephemerides, do NOT use this function in
% a parallel_map or other parallelized computations. This is
% because global caches are used that we cannot avoid if we
% want decent performance for single calls. Sorry.
%  
%\qualifiers{
% \qualifier{tdb}{The time argument is in TDB, not TT}
% \qualifier{mjd}{The time argument is in MJD, not in JD}
% \qualifier{barycentric}{Return coordinates with respect to the solar system barycenter rather
%                    than heliocentric coordinates.}
% \qualifier{spherical}{Return spherical coordinates in the ecliptical coordinate system.
%                       In this case the structure returned contains the elements
%                       lat, lon, r, latdot, londot, rdot, where lat and lon are the ecliptical
%                       latitude and longitude (in rad), r is the distance in AU, and
%                       latdot, londot, rdot are their time derivatives (in rad/day or
%                       AU/day, respectively.)}
% \qualifier{equatorial}{return coordinates in the ICRS2 coordinate system, not the
%                       equatorial one (vsop87 does not have this option)}
% \qualifier{mean_equinox}{Coordinates are given in the frame defined by the
%                   mean equinox and ecliptic of the date.
%                   The default is to return J2000 coordinates.}
% \qualifier{deg}{All angle quantities to be returned are in deg, or deg/day.}
% \qualifier{mks}{All distances returned by the function are given in meters or m/s, depending
%                 on the deg qualifier, angles are returned in rad and rad/s or deg and deg/s.}
% \qualifier{cgs}{Dito, but use centimeters instead of meters.}
% \qualifier{ephemeris}{Ephemeris structure initialized with jpl_initeph.}
% \qualifier{forcearray}{Force the tags in the returned struct to be arrays.
%                          The default is that they are arrays only if JD
%                          is an array.}
%}
%\seealso{vsop87,jpl_initeph,jpl_eph}
%!%-
{
    variable jdin,planet;
    switch(_NARGS)
    {
	case 2: (jdin,planet) = ();
    }
    { help(_function_name()); return; }

    % warn people if they expect the elements qualifier
    if (qualifier_exists("elements")) {
	throw UsageError,sprintf("%s: Elements qualifier is not supported\n",_function_name());
    }

    % if ephemeris is set, then pass it through
    variable eph;
    if (qualifier_exists("ephemeris")) {
	eph=qualifier("ephemeris");
    } else {
	eph=jpl_initeph();
    }
    
    % take care of JD being an array
    if (typeof(jdin)==Array_Type) {
	% initialize the ephemeris only once!
	variable qual=__qualifiers();
	if (not qualifier_exists("ephemeris")) {
	    qual=struct_combine(qual,struct {ephemeris=eph});
	}

	% we cannot use parallel map on jpl_eph_vec, so let's do this the
	% old style way
	variable res={};
	variable i;
	_for i(0,length(jdin)-1,1) {
	    list_append(res,jpl_eph_vec(jdin[i],planet;;qual));
	}
	return struct_array_2_struct_of_arrays(list_to_array(res));
    }
    
    % center body: default is heliocentric
    variable cb="Sun";
    if (qualifier_exists("barycentric")) {
	cb="SSB";
    }

    variable JD=@jdin;
    if (qualifier_exists("mjd")) {
	JD=mjd2jd(JD);
    }

    % input is TT, so find the corresponding TDB
    if (not qualifier_exists("tdb")) {
        % do conversion with ephemeris IF the ephemeris provides
        % the time conversion.  otherwise default to Bretagnon.
        if (length(eph.pars.numcoeff)>16 && eph.pars.numcoeff[16]>0) {
            JD=tt2tdb(JD;ephemeris=eph);
        } else {
            JD=tt2tdb(JD);
        }
    }

    % from here on time is in TDB

    % get xyz coordinates
    variable pos=jpl_eph(JD,planet,cb;ephemeris=eph);

    variable p=vector(pos[0],pos[1],pos[2]);
    variable v=vector(pos[3],pos[4],pos[5]);
    
    if (qualifier_exists("cgs")) {
    	p=p*1e5;
    	v=v*1e5;
    } else {
	if (qualifier_exists("mks")) {
     	    p=p*1000.;
     	    v=v*1000.;
	} else {
     	    p=p/eph.au;
     	    v=v*(86400./eph.au);
	}
    }

    % go to position for the equinox of date?
    if (qualifier_exists("mean_equinox")) {
	variable Bmat=gcrs2j2000_matrix(); % frame bias matrix
	variable Pmat=precession_matrix(JD);
	variable mat=Pmat*Bmat;
	p=mat*p;
	v=mat*v;
    }
    
    % go to ecliptical coordinate system?
    if (not qualifier_exists("equatorial")) {
	% conversion matrix from equatorial to ecliptical coordinates
	variable epsilon;
	if (qualifier_exists("mean_equinox")) {
	    epsilon=obliquity(JD);
	} else {
	    epsilon=obliquity(2451545.0);
	}
	variable tmat=matrix33_rot(1,epsilon);
	p=tmat*p;
	v=tmat*v;
    }

    % convert to angles?
    if (not qualifier_exists("spherical")) {
	if (qualifier_exists("forcearray")) {
	    return struct { x=[p.x], y=[p.y], z=[p.z], vx=[v.x],vy=[v.y],vz=[v.z] };
	} else {
	    return struct { x=p.x, y=p.y, z=p.z, vx=v.x,vy=v.y,vz=v.z };
	}
    }
    variable r,lon,lat;
    (r,lon,lat)=vector_to_spherical(p);

%    r=sqrt(p.x*p.x+p.y*p.y+p.z*p.z);
%    lat=asin(p.z/r);
%    lon=atan2(p.y,p.x);
    variable rdot=(v.x*p.x+v.y*p.y+v.z*p.z)/r;
    variable latdot=(r*v.z-p.z*rdot)/(r^2*sqrt((1.-(p.z/r))*(1.+(p.z/r))));
    variable londot=(p.x*v.y-p.y*v.x)/(p.x^2+p.y^2);
    
    if (qualifier_exists("deg")) {
	lat*=180./PI;
	lon*=180./PI;
	latdot*=180./PI;
	londot*=180./PI;
    }
    
    if (qualifier_exists("forcearray")) {
	return struct { lat=[lat], lon=[lon], r=[r], latdot=[latdot], londot=[londot], rdot=[rdot] } ;
    }
    return struct { lat=lat, lon=lon, r=r, latdot=latdot, londot=londot, rdot=rdot } ;
	
}

define planetpos();

define planetpos()
%!%+
%\function{planetpos}
%\synopsis{Calculate the apparent position of the Sun, Moon, and the planets
% (Mercury,..., Neptune, and Pluto).}
%\usage{(r,alp,del) = planetpos(JD;qualifiers);}
%\altusage{(r,alp,del) = planetpos(JD,object;qualifiers);}
%\description
%
% Returns the true geocentric distance (in AU), and apparent right ascension
% and declination of a planet in the geocentric celestial reference system for the
% date JD (in TT or TDB [the default]).
% 
% Use the center-qualifier to choose another center object.
%
% The default coordinate system is the J2000.0 (ICRS) system, to include the
% effects of precession and nutation to get "apparent" positions, use the
% of_date qualifier, to get the mean equinox position (w/o nutation), use
% of_date together with the mean_equinox qualifier.
%
% The true geocentric distance is the distance the planet has at the time of the
% calculation, while the apparent position is corrected for light travel time
% effects. This is the default (and identical to the Astronomical Almanac).
% If the lighttravel qualifier is set, the function returns the
% distance the light traveled. In most cases, this is what you want.
% Use the "no_apparent" to switch off the correction for light travel time
% (e.g., useful for the calculation of heliocentric coordinates, where
% traditionally light travel time effects are are not taken into account)
%
% The default is the JPL ephemeris DE430 (as used in the astronomical almanac
% and as distributed with HEASOFT). Use the ephemeris qualifier to switch to
% another ephemeris.
%
% Alternatively, the less precise VSOP87 can be used (but not for Pluto and
% the Moon). There is no computing time advantage to doing so, however, but
% vsop87 is valid for several 1000 years around AD2000, while most JPL
% ephemerides are limited to less than that.
%
% This function has been checked against the values tabulated in the
% Astronomical Almanac for 2014 and 2015, which also uses DE430.
% The results match exactly.
%
% JD and object can be arrays. In this case the return value is
% an array of Vectors or three arrays of distance, RA and DEC,
% sorted by object first and by date next.
%
%\qualifiers{
% \qualifier{object}{a string (valid values: Sun, Mercury, Venus,
%                      Mars, Jupiter, Saturn, Uranus, Neptune, Pluto, Moon,
%                      default: Sun).}
% \qualifier{center}{center object for the position. Default: Earth.
%                      Either a string (same as "object"), or a
%                      3D or 6D long double array. If 3D: position
%                      in AU in a barycentric coordinate system in
%                      ICRS coordinates (xyz), if 6D: position and
%                      velocity (AU/D) in a barycentric coordinate system.}
% \qualifier{topocentric}{return topocentric positions; if this qualifier
%                      is of type Vector_Type, then the xyz coordinates are
%                      interpreted as the position of the observer in the
%                      IERS 2010 coordinate system, in units of meters (see
%                      function geographic2vector). Otherwise, a structure with
%                      geographic coordinates is used, use tags lat, lon for
%                      the geographic position (in degrees, east is positive),
%                      and tag height as height (in meters) over the WGS 84
%                      geoid (as appropriate for GPS coordinates).}
% \qualifier{of_date}{if set, return the apparent position in the geocentric
%                       celestial reference system for the ecliptic and
%                       equinox of date (this is what is listed in the
%                       astronomical almanac).}
% \qualifier{mean_equinox}{if set together with of_date, return coordinates
%                       for the mean equinox of date (i.e., ignore nutation).}
% \qualifier{no_apparent}{ignore light travel time effects}
% \qualifier{ecliptical}{return coordinates in ecliptical coordinates,
%                        i.e., return r,lon,lat (or the xyz vector in ecliptical coords)
%                       (also takes of_date qualifier into account)}
% \qualifier{vector}{return a geocentric/object centric direction (!) vector
%                      for the position.}
% \qualifier{deg}{return the angles in degrees (default: radian).}
% \qualifier{mks}{return distance in m (default: AU)}
% \qualifier{cgs}{return distance in cm (default: AU)}
% \qualifier{lighttravel}{return distance the light traveled (default: true geocentric dist)}
% \qualifier{mjd}{the given date is the MJD}
% \qualifier{tt}{the given date is TT (default: TDB, difference only matters for pulsar work)}
% \qualifier{vsop87}{use the VSOP87 ephemeris rather than the JPL ephemeris, does not work
%                      for Pluto and the Moon.}
%}
%\seealso{jpl_initeph,vsop87,geographic2vector}
%!%-
%
{

    % fixme: avoid roundoff errors in time conversion
    % TODO: we should move the initializers for the ephemeris and tt
    % before the array calls, since this will save some time
    % 

    variable jdin,objects;

    switch (_NARGS)
    { case 1: jdin=(); }
    { case 2: (jdin,objects)=(); }
    { help(_function_name()); return;}
    
    ifnot (__is_initialized(&objects)) {
	objects=qualifier("object","Sun");
    } else {
	if (qualifier_exists("object")) {
	    throw UsageError,sprintf("%f: cannot give object as argument and qualifier at the same time\n",_function_name());
	}
    }

    %
    % Array arguments
    %
    variable i,j;
    if (typeof(jdin)==Array_Type || typeof(objects)==Array_Type) {
        variable qual=__qualifiers();
	% take care of case that objects were given via the qualifier
        if (typeof(qual)!=Null_Type) {
            if (struct_field_exists(qual,"object")) {
                qual=reduce_struct(qual,"object");
            }
        }

	% force objects to be an array (otherwise, if only one string
	% is given, the array syntax will give the ASCII value of the
	% first character in the string...)
	if (typeof(objects)!=Array_Type) {
	    objects=[objects];
	}
	
	if (qualifier_exists("vector")) {
	    variable res={};
	    _for i(0,length(jdin)-1,1) {
		_for j(0,length(objects)-1,1) {
		    list_append(res,planetpos(jdin[i],objects[j];;qual));
		}
	    }
	    return list_to_array(res);
	}

	variable res_r={};
	variable res_ra={};
	variable res_de={};
	variable r_r,r_ra,r_de;
	
	_for i(0,length(jdin)-1,1) {
	    _for j(0,length(objects)-1,1) {
		(r_r,r_ra,r_de)=planetpos(jdin[i],objects[j];;qual);
		list_append(res_r,r_r);
		list_append(res_ra,r_ra);
		list_append(res_de,r_de);
	    }
	}
	return (list_to_array(res_r),list_to_array(res_ra),list_to_array(res_de));
    }
    
    % from here, jdin and object are definitively scalars
    variable JD=jdin;
    if (qualifier_exists("mjd")) {
	JD=mjd2jd(JD);
    }

    variable planet=strup(objects);

    variable eph;
    if (qualifier_exists("ephemeris")) {
	eph=qualifier("ephemeris");
    } else {
	if (not qualifier_exists("vsop87")) {
	    % ephemeris not given: use the cached ephemeris or open an
	    % new one
	    % (NB: jpl_initeph with no argument defaults to DE430. It
	    % also does all of the error handling
	    % and also search in LHEA_DATA and LHEASOFT)
	    ifnot (__is_initialized(&jpl_ephsave)) {
		jpl_ephsave=jpl_initeph();
	    }
	    eph=jpl_ephsave;
	}
    }

    % we do all calculation in tdb rather than in tt
    % this means that when using vsop87 we convert back to tt, but
    % it makes the structure of the code here much simpler
    if (qualifier_exists("tt")) {
	JD=tt2tdb(JD;ephemeris=eph);
    } 

    variable centerobj=qualifier("center","Earth");
    variable cen;
    if (typeof(centerobj)==Array_Type) {
	if (length(centerobj)!=3 and length(centerobj)!=6) {
	    throw UsageError,sprintf("%s: center is vector and has invalid length\n",_function_name());
	}
	if (length(centerobj)==3) {
	    cen=[centerobj[0],centerobj[1],centerobj[2],0.,0.,0.];
	} else {
	    cen=[centerobj[0],centerobj[1],centerobj[2],centerobj[3],centerobj[4],centerobj[5]];
	}
	centerobj="POSITION";
    } else {
	centerobj=strup(centerobj);
    }

    if (centerobj==planet) {
	throw UsageError,sprintf("%s: Center object and target object cannot be identical.\n",_function_name());
    }

    % speed of light in au/day and GM/c^2 (in AU)
    variable c,muc2,auval;
    if (qualifier_exists("vsop87")) {
	c=173.1446;
	muc2=9.87e-9;
	auval=149597870.66; % value for the DE200 (i.e. DE118 per Standish, 1990)
    } else {
	c=86400.*eph.clight/eph.au;
	muc2=eph.gms/c/c;
	auval=eph.au;
    }

    
    %
    % decide on the ephemeris handlers
    %
    variable ephfunc;
    variable ephqual;
    if (qualifier_exists("vsop87")) {
	ephfunc=&vsop87;
	ephqual=struct {
	    tdb=1,
	    barycentric=1
	};
    } else {
	ephfunc=&jpl_eph_vec;
	ephqual=struct {
	    tdb=1,
	    equatorial=1,
	    ephemeris=eph
	};
    }


    %
    % topocentric position rather than planetocentric?
    %

    % relative position and velocity of observer wrt planet's
    % center
    variable g=vector(0.,0.,0.);
    variable gdot=vector(0.,0.,0.); 

    if (qualifier_exists("topocentric")) {
        if (centerobj!="EARTH") {
            throw UsageError,sprintf("%s: Computation of topocentric positions is currently only possible for Earth.\n",_function_name()); 
        }

        variable topopos;
        if (typeof(qualifier("topocentric"))==Vector_Type) {
            topopos=qualifier("topocentric"); % xyz in m
        } else {
            if (typeof(qualifier("topocentric"))==Struct_Type) {
                variable tp=qualifier("topocentric");
                topopos=geographic2vector(tp.lon,tp.lat,tp.height;deg,datum="WGS 84");
            } else {
                throw UsageError,sprintf("%s: Topocentric coordinates must be given as Vector or struct\n");
            }
        }

        %
        % compute observer's offset from Earth's center
        variable ut1=tt2ut1(tdb2tt(JD));
        variable beta=GMST(ut1;gast)/86400. * 2.*PI; % rad
        variable betadot=7.292115e-5; % rad/s (angular rotation of Earth)

        % geocentric position of the observer, ignoring
        % polar motion (per expl. suppl., p 296, this is
        % at the mas level for the moon, and smaller for
        % all other objects...)
        variable sb,cb;
        (sb,cb)=sincos(beta);
        g=matrix33_rot(3,-beta)*topopos;
        gdot=betadot * matrix33_new(-sb,-cb,0.,cb,-sb,0.,0.,0.,0.) * topopos;
        % m -> au  / m/s -> au/day
        g=g/(auval*1000.);
        gdot=gdot*86400./(auval*1000.);
    }
    
    variable p2; % will contain the geocentric position at the end
    variable ebary,sbary,qbary,ehelio;
    variable truedist=0.;
    
    if (planet!="SUN" and centerobj!="SUN") {
	% barycentric position of the center object and the Sun
	if (centerobj=="POSITION") {
	    ebary=struct {x=cen[0],y=cen[1],z=cen[2],vx=cen[3],vy=cen[4],vz=cen[5]};
	} else {
	    ebary=(@ephfunc)(JD,centerobj;;ephqual);
	}

	sbary=(@ephfunc)(JD,"Sun";;ephqual);

        % barycentric position and velocity of the Earth
	variable Eb=vector(ebary.x,ebary.y,ebary.z);
	variable Ev=vector(ebary.vx,ebary.vy,ebary.vz);
        
	% barycentric position of the Sun now
	variable Sb0=vector(sbary.x,sbary.y,sbary.z);

        % correction for topocentric positions
        % (g and gdot are 0 if we remain geocentric)
        Eb+=g;
        Ev+=gdot;
        
	% heliocentric position of the Earth now
	variable E=Eb-Sb0;
	variable ee=abs(E);
	
	%
	% adjust position for the light travel time
	% in an iterative way
	%
	variable tau=0.;
	do {

	    % Vector from the center to the planet
	    qbary=(@ephfunc)(JD-tau,planet;;ephqual);
	    variable Qb=vector(qbary.x,qbary.y,qbary.z);
	    variable P=Qb-Eb;
	    variable pp=abs(P);
	    if (truedist==0.) { % remember true geocentric distance
		truedist=pp;
	    }

	    % barycentric position of the Sun
	    sbary=(@ephfunc)(JD-tau,"Sun";;ephqual);
	    variable Sb=vector(sbary.x,sbary.y,sbary.z);

	    % heliocentric position of the planet
	    variable Q=Qb-Sb;
	    variable qq=abs(Q);
	    
	    % light travel time, including gravitational field of the sun
	    % (Shapiro delay [except for case of the Sun])
	    variable tauold=tau;
	    tau=(pp+2.*muc2*log((ee+pp+qq)/(ee-pp+qq)))/c;
	    if (qualifier_exists("no_apparent")) {
		tauold=tau;
	    }

	} while ( abs(tau-tauold) > 1e-9);
	    
	%
	% correction for relativistic deflection of light
	%

	% unit vectors
	variable pnorm=P/abs(P);
	variable qnorm=Q/abs(Q);
	variable enorm=E/ee;
	    
	variable g1=2*muc2/ee;
	variable g2=1.+qnorm*enorm;

	% geocentric direction of the planet, corrected for light
	% deflection due to the Sun
	variable p1=pnorm+g1/g2 * ((pnorm*qnorm)*enorm - (enorm*pnorm)*qnorm); 
	    
	%
	% aberration of light
	% (relativistic calculation, which is overkill for most applications)
	%
	variable vel=Ev*(1./c);
	variable vv=abs(vel);
    	variable beta1=sqrt((1.-vv)*(1.+vv)); % 1/beta

	variable f1=p1*vel;
	variable f2=1.+f1/(1.+beta1);
	
	p2=(beta1*p1+ f2*vel)/(1.+f1);
    } else {
	%
	% special treatment we are either calculating
	% heliocentric coordinates or the position of the
	% Sun
	%
	variable ccobj=centerobj;
	if (ccobj=="SUN") {
	    ccobj=planet;
	}

        tau=0.;
	% work around a s-lang problem with function pointers apparently not
	% allowing named qualifiers
	variable ephqual2=struct_combine(ephqual,struct{heliocentric=1});
	do {
	    ehelio=(@ephfunc)(JD-tau,ccobj;;ephqual2);
            variable Eh=vector(ehelio.x,ehelio.y,ehelio.z)+g;
	    tauold=tau;
	    if (truedist==0.) {
		truedist=abs(Eh);
	    }
	    tau=abs(Eh)/c;
	    if (qualifier_exists("no_apparent")) {
		tauold=tau;
	    }
	} while (abs(tau-tauold) > 1e-9);
	if (ccobj=="SUN") {
	    p2=Eh;
	} else {
	    p2=-Eh;
	}
    }

    if (qualifier_exists("vsop87")) {
	% conversion matrix from ecliptical to equatorial coordinates
	% (Bretagnon & Francou, 1988, eq 1)
	%
	% NB because of the phi rotation this is slighly different than the
	% rotation back to ecliptical below, 
	% so we will always do this 
	variable eps=dms2deg(23,26,21.4091;radian);
	variable phi=dms2deg(0.,0.,-0.0930;radian);

	variable ecl2icrf=matrix33_rot(3,phi)*matrix33_rot(1,-eps);
    
	p2=ecl2icrf*p2; % vector is now in the GCRF
    }

    % we now have the GCRS position
    
    % 
    % convert GCRS position into
    % position for the equinox of date?
    %
    variable p3;
    if (qualifier_exists("of_date")) {
	variable Bmat=gcrs2j2000_matrix(); % frame bias matrix
	variable Pmat=precession_matrix(JD);

	if (qualifier_exists("mean_equinox")) {
	    p3=Pmat*Bmat*p2;
	} else {
	    variable Nmat=nutation_matrix(JD);
	    p3=Nmat*Pmat*Bmat*p2;
	}
    } else {
	p3=p2;
    }

    % this is faster than using equatorial2ecliptical
    if (qualifier_exists("ecliptical")) {
	variable epsilon;
	if (qualifier_exists("of_date")) {
	    if (qualifier_exists("mean_equinox")) {
		epsilon=obliquity(JD);
	    } else {
		epsilon=obliquity(JD;true);
	    }
	} else {
	    epsilon=obliquity(2451545.0;true);
	}
	variable tmat=matrix33_rot(1,epsilon);
	p3=tmat*p3;
    }
    
    if (qualifier_exists("vector")) {
	% return vector quantity
	return p3; 
    }

    % default: RA, and DEC
    variable r,alp,del;

    if (qualifier_exists("deg")) {
	(r,alp,del)=vector_to_spherical(p3;deg);
    } else {
	(r,alp,del)=vector_to_spherical(p3);
    }

    if (qualifier_exists("lighttravel")) {
	truedist=tau*c;
    }

    if (qualifier_exists("mks")) {
	truedist*=auval*1000.;
    }

    if (qualifier_exists("cgs")) {
	truedist*=auval*1e5;
    }
    
    return (truedist,alp,del);
}
