define mass_function()
%!%+
%\function{mass_function}
%\synopsis{Calculates the mass function of a binary.}
%\usage{Double_Type   mass = mass_function(Double_Type porb, asini[, error_porb, error_asini]);
% or Double_Type[]    i = mass_function(Double_Type mass, Double_Type[] Mopt[, error_mass, error_Mopt]; i);}
%\qualifiers{
%    \qualifier{i}{calculate and return the inclination (case 2a)}
%    \qualifier{Mx}{neutron star mass in units of the solar mass
%             (default: 1.4)}
%    \qualifier{rad}{return the inclination angle in radian
%             (default: degrees)}
%    \qualifier{chatty}{set to zero to suppress output messages}
%}
%\description
%    The mass function of a binary is given by
%    (see, e.g., Hilditch, 2001)
%
%    f(M) = (M_opt sin i)^3 / (M_x + M_opt)^2
%         = (4 Pi^2 (a sin i)^3) / (G P_orb^2)
%
%    with the masses of the neutron star, M_x, and its optical
%    ompanion star, M_opt, the inclination, i, the semi-major axis,
%    a, the orbital period, P_orb, and the gravitational constant, G.
%
%    There are two ways this function can be used:
%    1)  The orbital period (in days) and the semi-major axis (in
%        lt-s) are provided, which then is used to calculate the
%        right hand side of the above equation. The returned mass is
%        given in units of the solar mass.
%    2)  The value of the mass function, mass, and the companion
%        mass(es), Mopt, are provided and the i-qualifier is set.
%        The returned value(s) is (are) the orbital inclination(s), i.
%    In case of 2) the neutron star mass is assumed to be Mx = 1.4
%    solar masses by default, but can be changed by the Mx-qualifier.
%    The thirs possible case, that the companion mass is returned,
%    is formally solving a third order polynomial, which is yet not
%    implemented here.
%
%    If uncertainties for the given parameters are provided the error
%    propagated values are returned as well for the specific case via
%      (value,error) = mass_funtion(..., error(s));
%!%-
{
  variable arg1, arg2, err1 = NULL, err2 = NULL;
  switch(_NARGS)
  { case 2: (arg1, arg2) = (); }
  { case 4: (arg1, arg2, err1, err2) = (); }
  { help(_function_name()); return; }

  % qualifiers
  variable Mx = qualifier("Mx", 1.4);
  variable chatty = qualifier("chatty", 1);
  variable errprop = err1 != NULL && err2 != NULL;

  % case
  variable icase = qualifier_exists("mopt") ? 3
                 : qualifier_exists("i") ? 2 : 1;
  variable value, err;
  switch (icase) 
  % porb and asini given, case 1
  { case 1:
    if (chatty) { message("orbital period and semi-major axis given -> returning mass function"); }
    value = 1.0735e-3 * arg2^3 / arg1^2;
    if (errprop) {
      err = 1.0735e-3 * sqrt(sqr(3*arg2^2/arg1^2 * err2) + sqr(2*arg2^3/arg1^3 * err1));
    }
  }
  % companion mass given, case 2
  { case 2:
    if (chatty) { message("companion mass given, returning inclination"); }
    value = asin(((Mx + arg2)^2 * arg1)^(1./3) / arg2);
    ifnot (qualifier_exists("rad")) { value = value*180./PI; }
    if (errprop) {
      variable c = 3*arg2*sqrt(1 - (arg1^(2./3)*(Mx+arg2)^(4./3))/arg2^2);
      err = sqrt(
        sqr((2*arg2*(arg1/(Mx+arg2))^(1./3) - 3*arg1^(1./3)*(Mx+arg2)^(2./3))
	   /(arg2*c) * err2)
      + sqr(((Mx+arg2)/arg1)^(2./3)/c * err1)
      );
    }
  }
  { case 3:
    if (chatty) { message("inclination given, returning companion mass"); }
    message("not implemented yet");
  }

  if (errprop) { return (value, err); }
  else { return value; }
}
