define obliquity ()
%!%+
%\function{obliquity}
%\synopsis{calculate the obliquity of the ecliptic}
%\usage{eps=obliquity(JD;mjd);}
%\qualifiers{
%  \qualifier{mjd}{if set, the argument is in MJD, not in JD}
%  \qualifier{laskar}{use the polynomial expression by Laskar (1986,
%            Astron. Astrophys. 157, 59), which is valid for a time
%            span of around 10000 years around J2000, rather than
%            the IAU 2006 expression.}
%  \qualifier{true}{return the true obliquity for the date by applying
%            nutation}
%  \qualifier{theory}{nutation theory to use, see help for
%            function nutation_angles; default: NU2000K}
%  \qualifier{deg}{return the obliquity in degrees }
%  \qualifier{arcsec}{return the obliquity in arcseconds}
%}
%\description
% This routine calculates the obliquity of the ecliptic for
% the Julian Date(s) JD using the IAU 2006 resolutions, which
% are based on Hilton et al., (2006, Cel. Mech. Dyn. Astron. 94, 351).
% JD can be an array, it is formally measured in TT.
%
% The IAU 2006 expression is good to better than 1 arcsecond
% for +/-500 years around J2000. Outside of this interval
% use the polynomial expression by Laskar (laskar keyword).
%
% The angles returned are in rad, use the deg or arcsec qualifiers
% if you want them in degrees or arc seconds.
%
%\seealso{nutation_angles}
%!%-
{
    variable JD,T;
    switch(_NARGS)
    {
	case 1: JD=();
    }
    {help(_function_name()); return; }

    if (qualifier_exists("mjd")) {
	JD=mjd2jd(JD);
    }

    if (qualifier_exists("deg") && qualifier_exists("arcsec")) {
	throw UsageError, sprintf("Usage error in %s: Only one of 'deg' and 'arcsec' qualifiers is allowed\n",_function_name());
    }

    % time in centuries since J2000.0
    T=(JD-2451545.0)/36525.;
    
    variable eps;
    
    if(qualifier_exists("laskar")) {
	% Laskar (1986, Table 8; T is per 10000 yr!)
	T=T/100.;
	eps=(((((((((2.45*T+5.79)*T+27.87)*T+7.12)*T-39.05)*T-249.67)*T-51.38)*T+1999.25)*T-1.55)*T-4680.93)*T;
    } else {
	% IAU 2006 equations
	% (T is per century!)
	eps=((((-4.34E-8*T-5.76E-7)*T+0.00200340)*T-0.0001831*T)-46.836769)*T;
    }
    eps+=84381.406;

    if (qualifier_exists("true")) {
	variable dpsi,deps;
	(dpsi,deps)=nutation_angles(JD;theory=qualifier("theory","NU2000K"),arcsec);
	eps+=deps;
    }
    
    if (qualifier_exists("arcsec")) {
	return eps;
    }

    if(qualifier_exists("deg")) {
	return eps/3600.;
    }

    return eps*PI/648000.; % arcsec -> rad
}
