define properties_satellite_galaxies()
%!%+
%\function{properties_satellite_galaxies}
%\synopsis{Retrieve the properties of several satellite galaxies of the Milky Way}
%\usage{Struct_Type s = properties_satellite_galaxies()}
%\description
%    Retrieve the properties (current kinematics, spatial extent, mass) of several satellite
%    galaxies of the Milky Way.
%
%    The output structure contains for each object the celestial coordinates in right ascension
%    [h, m, s] and declination [deg, arcmin, arcsec], distance [kpc], radial velocity [km/s],
%    proper motion in right ascension times cosine of declination [mas/yr], proper motion in
%    declination [mas/yr]), half-light radius [kpc], dynamcial mass inside half-light radius
%    [solar masses], Plummer softening radius [kpc], and the Plummer mass [solar masses].
%\notes
%    Positions, distances, radial velocities, half-light radii, and dynamcial masses are from
%    McConnachie 2012 when available. Proper motions (PMs) are available only for a subset of
%    satellites and are set to zero otherwise.
%
%    The parameters of the Plummer potentials (softening radius, mass) are determined under
%    the assumption that the dynamcial mass inside the half-light radius is 0.85 times the
%    total mass, which gives a softening radius of about one third of the half-light radius.
%
%    Main reference: McConnachie 2012, AJ, 144, 4.
%    Additional references or comments:
%    - LMC, SMC:         PMs and Plummer parameters from Kallivayalil et al. 2013, ApJ, 764, 161.
%                        Half-light radius and mass derived from Plummer parameters.
%    - Canis Major:      PMs from Dinescu et al. 2005, AJ, 631, L49.
%                        Dynamical mass assumed to be ten times stellar mass, half-light radius set to 1 kpc.
%    - Sagittarius dSph: PMs from Pryor et al. 2010, AJ, 139, 839.
%    - Fornax:           PMs from Piatek et al. 2007, AJ, 133, 818.
%    - Sextans:          PMs from Walker et al. 2008, ApJ, 688, L75.
%    - Sculptor:         PMs from Piatek et al. 2006, AJ, 131, 1445.
%    - Draco:            PMs from Pryor et al. 2014, AJ, submitted [arXiv:1407.3509].
%    - Ursa Minor:       PMs from Piatek et al. 2005, AJ, 130, 95.
%    - Carina:           PMs from Piatek et al. 2003, AJ, 126, 2346.
%    - Leo I:            PMs from Sohn et al. 2013, ApJ, 768, 139.
%    - Leo II:           PMs from Lepine et al. 2011, ApJ, 741, 100.
%    - Bootes III:       Dynamical mass assumed to be ten times stellar mass, half-light radius set to 10 pc.
%\example
%    s = properties_satellite_galaxies();
%    print_struct(s);
%\seealso{N_body_simulation_MW_kernel}
%!%-
{
  if(_NARGS!=0)
  {
    % _pop_n(_NARGS); % remove function arguments from stack
    help(_function_name());
    throw UsageError, sprintf("Usage error in '%s': Wrong number of arguments.", _function_name());
  }
  %
  variable s = struct{ object, ah, am, as, dd, dm, ds, dist, vrad, pma_cos_d, pmd, hl_radius, hl_mass, Pl_radius, Pl_mass };
  s.object    = [    "LMC",    "SMC", "CanisMajor", "SagittariusdSph", "Fornax", "Sextans", "Sculptor",  "Draco", "UrsaMinor", "Carina",   "LeoI", "LeoII", "Segue", "UrsaMajorII", "BootesII", "SegueII", "Willman1",
		 "ComaBerenices", "Bootes", "UrsaMajor", "Hercules", "LeoIV", "CanesVenatici", "CanesVenaticiII", "BootesIII"];
  s.ah        = [    5.   ,    0.   ,         7.  ,             18.  ,    2.   ,     10.  ,       1.  ,   17.   ,       15.  ,     6.  ,  10.    ,  11.   ,   10.  ,          8.  ,      13.  ,      2.  ,      10.  ,
		 12.  ,    14.  ,       10.  ,      16.  ,   11.  ,           13.  ,             12.  ,       13.  ];
  s.am        = [   23.   ,   52.   ,        12.  ,             55.  ,   39.   ,     13.  ,       0.  ,   20.   ,        9.  ,    41.  ,   8.    ,  13.   ,    7.  ,         51.  ,      58.  ,     19.  ,      49.  ,
		 26.  ,     0.  ,       34.  ,      31.  ,   32.  ,           28.  ,             57.  ,       57.  ];
  s.as        = [   34.5  ,   44.8  ,        35.  ,             19.5 ,   59.3  ,      3.  ,       9.4 ,   12.4  ,        8.5 ,    36.7 ,  28.1   ,  28.8  ,    4.0 ,         30.  ,       0.  ,     16.  ,      21.  ,
  		 59.  ,     6.  ,       52.8 ,       2.  ,   57.  ,            3.5 ,             10.  ,       12.  ];
  s.dd        = [  -69.   ,  -72.   ,       -27.  ,            -30.  ,  -34.   ,     -1.  ,     -33.  ,   57.   ,       67.  ,   -50.  ,  12.    ,  22.   ,   16.  ,         63.  ,      12.  ,     20.  ,      51.  ,
		 23.  ,    14.  ,       51.  ,      12.  ,    0.  ,           33.  ,             34.  ,       26.  ];
  s.dm        = [   45.   ,   49.   ,        40.  ,             32.  ,   26.   ,     36.  ,      42.  ,   54.   ,       13.  ,    57.  ,  18.    ,   9.   ,    4.  ,          7.  ,      51.  ,     10.  ,       3.  ,
  		 54.  ,    30.  ,       55.  ,      47.  ,  -32.  ,           33.  ,             19.  ,       48.  ];
  s.ds        = [   22.   ,   43.   ,         0.  ,             43.  ,   57.   ,     53.  ,      33.  ,   55.   ,       21.  ,    58.  ,  23.    ,   6.   ,   55.  ,         48.  ,       0.  ,     31.  ,       0.  ,
  		 15.  ,     0.  ,       12.  ,      30.  ,    0.  ,           21.  ,             15.  ,        0.  ];
  s.dist      = [   51.   ,   64.   ,         7.  ,             26.  ,  147.   ,     86.  ,      86.  ,   76.   ,       76.  ,   105.  , 254.    , 233.   ,   23.  ,         32.  ,      42.  ,     35.  ,      38.  ,
		 44.  ,    66.  ,       97.  ,     132.  ,  154.  ,          218.  ,            160.  ,       47.  ];
  s.vrad      = [  262.2  ,  145.6  ,        87.  ,            140.  ,   55.3  ,    224.2 ,     111.4 , -291.   ,     -246.9 ,   222.9 , 282.5   ,  78.   ,  208.5 ,       -116.5 ,    -117.  ,    -39.2 ,     -12.3 ,
		 98.1 ,    99.  ,      -55.3 ,      45.2 ,  132.3 ,           30.9 ,           -128.9 ,      197.5 ];
  s.pma_cos_d = [    1.910,    0.772,        -1.61,             -2.75,    0.476,     -0.26,       0.09,    0.177,       -0.50,     0.22,  -0.1140,   0.104,    0.  ,          0.  ,       0.  ,      0.  ,       0.  ,
		 0.  ,     0.  ,        0.  ,       0.  ,    0.  ,            0.  ,              0.  ,        0.  ];
  s.pmd       = [    0.229,   -1.117,         0.84,             -1.65,   -0.360,      0.10,       0.02,   -0.221,        0.22,     0.15,  -0.1256,  -0.033,    0.  ,          0.  ,       0.  ,      0.  ,       0.  ,
		 0.  ,     0.  ,        0.  ,       0.  ,    0.  ,            0.  ,              0.  ,        0.  ];
  s.hl_radius = [   24.   ,    3.   ,         1.  ,              2.6 ,    0.7  ,      0.7 ,       0.28,    0.22 ,        0.18,     0.25,   0.25  ,   0.18 ,    0.03,          0.15,       0.05,      0.04,       0.03,
		 0.08,     0.24,        0.32,       0.33,    0.21,            0.56,              0.08,        0.01];
  s.hl_mass   = [25500.   , 2550.   ,       490.  ,            190.  ,   56.   ,     25.  ,      14.  ,   11.   ,        9.5 ,     6.3 ,  12.    ,   4.6  ,    0.26,          3.9 ,       3.3 ,      0.23,       0.27,
		 0.94,     0.81,       11.  ,       2.6 ,    1.3 ,           19.  ,              0.91,        0.17]*10^6;
  s.Pl_radius = s.hl_radius/3.;
  s.Pl_mass = s.hl_mass/0.85;
  return s;
}
