define solve_Eddington_quartic_equation() %{{{
%!%+
%\function{solve_Eddington_quartic_equation}
%\synopsis{Solve the Eddington quartic equation}
%\usage{Double_Type beta = solve_Eddington_quartic_equation(Double_Type M, mu)}
%\description
%    For a given mass 'M' (in solar masses) and mean molecular
%    weight 'mu', solve the Eddington quartic equation
%      1-beta = 0.003*M^2*mu^4*beta^4
%    numerically for 'beta' by using Newton's method to find
%    the root of a function.
%\notes
%    According to Equation (2-15) in the book "Principles of
%    Stellar Evolution and Nucleosynthesis" by D.D. Clayton,
%    the mean molecular weight can be approximated by
%      mu ~ 2/(1+3*X+0.5*Y)
%    where 'X' and 'Y' are the mass fractions of hydrogen
%    and helium.
%
%    The Eddington quartic equation needs to be solved to
%    compute the polytropic constant of the polytropic standard
%    model, see the function 'polytropic_standard_model'.
%\example
%    X = 0.73; Y = 0.26; mu = 2/(1+3*X+0.5*Y);
%    beta = solve_Eddington_quartic_equation(1, mu);
%\seealso{polytropic_standard_model}
%!%-
{
  if(_NARGS!=2)
  {
    _pop_n(_NARGS); % remove function arguments from stack
    help(_function_name());
    throw UsageError, sprintf("Usage error in '%s': Wrong number of arguments.", _function_name());
  }
  variable M, mu;
  (M, mu) = ();
  variable beta_old = 0, beta = 1; % initial guess for beta
  variable M2mu4 = 0.003*M^2*mu^4;
  % Newton method for finding the root of the function f = 0.003*M^2*mu^4*beta^4 + \beta - 1
  while(abs(beta_old-beta)>1e-10)
  {
    beta_old = beta;
    beta = beta_old - (M2mu4*beta_old^4+beta_old-1.)/(4.*M2mu4*beta_old^3+1.);
  }
  return beta;
}%}}}
