define sunpos () 
%!%+
%\function{sunpos}
%\synopsis{computes the RA and Dec of the Sun at a given date.}
%\usage{(Double_Type eq, Double_Type ecl) = sunpos(Double_Type JD);}
%\description
%   Computes the position of the Sun at the date JD (or dates, if JD
%   is an array), given in Julian Date. Position calculation takes
%   into account pertubations from the planets, aberration and
%   nutation.  Output value 'eq' is a structure in equatorial
%   coordinates: eq = struct{ra, dec}, 'ecl' is in ecliptic plane
%   coordinates: ecl = struct{lon, obl} (Longitude and true
%   obliquity).
%   
%   Note: Function adopted from the IDL function with the same name.
%   See there for accuracy tests, which should also apply here.
%\qualifiers{
%\qualifier{MJD}{give time in MJD instead of JD}
%\qualifier{radian}{return all variables as radians rather than degrees}
%\qualifier{hour}{return RA in hours rather than degrees (not possible when radian is set).}
%}
%\example
%   (eq, ecl) = sunpos(2455055.5) ;
%   ()=printf("Right ascension: %.4f deg, Declination: %.4f deg\n", eq.ra, eq.dec) ;
%!%-
{
   variable jd   ;

   switch(_NARGS) 
     { case 1: (jd) = () ; }
     { help(_function_name()); return; }

    if (qualifier_exists("mjd") or qualifier_exists("MJD")) jd+=2400000.5  ; % translate MJD to JD
    
    variable dtor = PI/180.0  ; %       ;(degrees to radian, double precision)
  
  % ;  form time in Julian centuries from 1900.0
   variable t = (jd - 2415020.0)/36525.0 ;
  
  % ;  form sun's mean longitude
   variable l = (279.696678+((36000.768925*t) mod 360.0))*3600.0 ;

% ;  allow for ellipticity of the orbit (equation of centre)
% ;  using the Earth's mean anomaly ME
   variable me = 358.475844 + ((35999.049750*t) mod 360.0) ;
   variable ellcor  = (6910.1 - 17.2*t)*sin(me*dtor) + 72.3*sin(2.0*me*dtor) ;
     
   l = l + ellcor ;

% ; allow for the Venus perturbations using the mean anomaly of Venus MV
  variable mv = 212.603219 + ((58517.803875*t) mod 360.0) ; 
   
  variable vencorr = 4.8 * cos((299.1017 + mv - me)*dtor) + 
          5.5 * cos((148.3133 +  2.0 * mv  -  2.0 * me )*dtor) + 
          2.5 * cos((315.9433 +  2.0 * mv  -  3.0 * me )*dtor) + 
          1.6 * cos((345.2533 +  3.0 * mv  -  4.0 * me )*dtor) + 
          1.0 * cos((318.15   +  3.0 * mv  -  5.0 * me )*dtor) ;

   l = l + vencorr ;

% ;  Allow for the Mars perturbations using the mean anomaly of Mars MM
   variable mm = 319.529425  +  (( 19139.858500 * t)  mod  360.0 ) ;
 
   variable marscorr = 2.0 * cos((343.8883 -  2.0 * mm  +  2.0 * me)*dtor ) + 
            1.8 * cos((200.4017 -  2.0 * mm  + me) * dtor) ;
 
   l = l + marscorr ;

% ; Allow for the Jupiter perturbations using the mean anomaly of
% ; Jupiter MJ
   variable mj = 225.328328  +  (( 3034.6920239 * t)  mod  360.0 ) ;
 
   variable jupcorr = 7.2 * cos(( 179.5317 - mj + me )*dtor) + 
          2.6 * cos((263.2167  -  mj ) *dtor) + 
          2.7 * cos(( 87.1450  -  2.0 * mj  +  2.0 * me ) *dtor) + 
          1.6 * cos((109.4933  -  2.0 * mj  +  me ) *dtor) ;
 
   l = l + jupcorr ;

% ; Allow for the Moons perturbations using the mean elongation of
% ; the Moon from the Sun D
   variable d = 350.7376814  + (( 445267.11422 * t)  mod  360.0 ) ;
   variable mooncorr  = 6.5 * sin(d*dtor) ;
  
   l = l + mooncorr ;

% ; Allow for long period terms
   variable longterm  = + 6.4 * sin(( 231.19  +  20.20 * t )*dtor) ;
   l  =    l + longterm ;
   l  =  ( l + 2592000.0)  mod  1296000.0 ;
   variable longmed = l/3600.0 ;

% ; Allow for Aberration
   l  =  l - 20.5 ;

% ; Allow for Nutation using the longitude of the Moons mean node OMEGA
   variable omega = 259.183275 - (( 1934.142008 * t ) mod 360.0 ) ;
   l  =  l - 17.2 * sin(omega*dtor) ;

% ; Form the True Obliquity
   variable oblt  = 23.452294 - 0.0130125*t + (9.2*cos(omega*dtor))/3600.0 ;

% ; Form Right Ascension and Declination
   l = l/3600.0 ;

   %forcing RA to be an array, so that indexing works always
   variable ra  = [atan2( (sin(l*dtor) * cos(oblt*dtor)) , (cos(l*dtor)) )] ;
   
   variable neg = where(ra < 0.0) ;
   
   if (length(neg)) ra[neg] += 2.0*PI ;

   variable dec = asin(sin(l*dtor) * sin(oblt*dtor)) ;
 
   if(qualifier_exists("radian"))
     {
	oblt *= dtor ; longmed *= dtor ;
     }
   else 
     {
	if(qualifier_exists("hour")) ra /= 15. ; %transfer RA degrees to hours, only if radian is not set
	ra /= dtor ;  dec /= dtor ;
     }

   %returing RA to a single double value if array has only one entry.
   if (length(ra) == 1) { return struct{ra=ra[0], dec=dec[0]}, struct{lon=longmed[0], obl=oblt[0]} ; }
   else { return struct{ra=ra, dec=dec}, struct{lon=longmed, obl=oblt} ;} 
}
