private define print_ephem_data(ephem_data)
{
  variable ephem, field;
  foreach ephem (ephem_data)
  {
    ()=printf("%s :", ephem.name);
    variable comma = "";
    variable fields = get_struct_field_names(ephem);
    foreach field (fields[where(fields!="name")])
    {
      variable value =  get_struct_field(ephem, field);
      if(typeof(value)==String_Type)  value = make_printable_string(value);
      ()=printf("%s %s=%S", comma, field, value);
      comma = ",";
    }
    ()=printf("\n");
  }
}


define get_ephemeris()
%!%+
%\function{get_ephemeris}
%\synopsis{returns the epoch and period of an ephemeris}
%\usage{Struct_Type get_ephemeris(String_Type ephemeris)}
%\description
%    The list of known ephemerides is stored internally
%    and is shown if an unknown ephemeris is requested.
%    Ephemerides are returned as a structure, containing
%    at least the internal \code{name} of the ephemeris (which
%    usually contains the source name and a reference),
%    the epoch \code{T0}, and the period \code{P}.
%    More information may be stored in further reasonably named
%    fields, e.g., \code{Pdot} for the change of the period.
%\seealso{orbitalphase}
%!%-
{
  if(_NARGS!=1)  return help(_function_name());
  variable ephem_name = ();
  
  variable ephem_data = [
    struct{ name="CygX-1_Gies2008",         T0=52872.288,    P=  5.599829, T0_err=0.009 },
    struct{ name="CygX-1_Gies2003",         T0=51729.949,    P=  5.599829, T0_err=0.008 },
    struct{ name="CygX-1_Brocksopp1999",    T0=41874.207,    P=  5.599829, T0_err=0.009,  P_err=0.000016 },
    struct{ name="CygX-1_Lachowicz2006",    T0=50514.59,     P=151.43, note="superorbital modulation" },
    struct{ name="LMCX-1_Orosz2008",        T0=53390.8436,   P=  3.90917,  T0_err=0.0120, P_err=0.00005, reference="http://adsabs.harvard.edu/abs/2009ApJ...697..573O, Table 3, adopted value" },
	struct{ name="LMCX-4_Falanga2015",      T0=53013.5910,   P= 1.4083790,  T0_err = 0.0008, P_err = 7e-7},
	struct{ name="VelaX-1_Kreykenbohm2008", T0=52974.227,    P=  8.964386, T0_err=0.007,  P_err=0.000015 },
	struct{ name="VelaX-1_Falanga2015",     T0=42611.349,    P= 8.964427,   T0_err = 0.013,  P_err = 1.2e-5},
    struct{ name="CygX-2_Casares2010",      T0=51386.648,    P=  9.8445,   T0_err=0.018,  P_err=0.00019 },
    struct{ name="CygX-2_Casares1998",      T0=49339.00,     P=  9.8444,   T0_err=0.03,   P_err=0.0003 },
    struct{ name="CenX-3_Raichur2010",      T0=50506.788423, P=  2.08713936, Pdot=1.799e-6/365.25, Pddot=0 },  % be aware of \dot Porb
	struct{ name="CenX-3_Falanga2015",      T0=50506.788423, P= 2.08704106, T0_err = 7e-6, P_err = 3e-8},
	struct{ name="4U1907_Inam2009",         T0=54467.6,      P=  8.3753, asini= 83.0, eccentricity=0.28, omega=300 },
    struct{ name="4U0115_Mueller2010",      T0=49279.2677,   P= 24.31617 },
    struct{ name="IGR_J19140_Corbet2004",   T0=51593.4,      P= 13.558, note="orbit measured only from ASM data" },
    struct{ name="GX301-2_Koh1997",         T0=48802.79,     P= 41.498, asini=368.3, eccentricity=0.462, omega=310.4},
    struct{ name="4U1909_Levine2004",       T0=52631.383,    P= 4.4007, asini=47.83, eccentricity=0.021},
    struct{ name="4U1700_Falanga2015",      T0=53785.850,    P= 3.411581,   T0_err = 0.007,  P_err = 7e-6},
	struct{ name="4U1538_Falanga2015",      T0=52855.061,   P= 3.7284140,  T0_err = 0.013,  P_err = 7.6e-6},
	struct{ name="SMCX-1_Falanga2015",      T0=52846.688810, P= 3.891923160,T0_err = 2.4e-5, P_err = 6.6e-8},
	struct{ name="SAXJ1802_Falanga2015",    T0=52168.245,    P= 4.5697,     T0_err = 0.034,  P_err = 0.0001},
	struct{ name="XTEJ1855_Falanga2015",    T0=52704.009,    P= 6.07415,    T0_err = 0.017,  P_err = 8e-5},
	struct{ name="EXO1722_Falanga2015",     T0=53761.695,    P= 9.74079,    T0_err = 0.019,  P_err = 8e-5},
	struct{ name="OAO1657_Falanga2015",     T0=52674.119,    P=10.447355,   T0_err = 0.0017, P_err = 9.2e-5}
   ];

  variable j, i = {};  % ephem_name=="" shall be equivalent to an unknown ephemeris
  if(ephem_name != "")
    _for j (0, length(ephem_data)-1, 1)
      if(string_match(ephem_data[j].name, ephem_name))
	list_append(i, j);

  if(length(i))
  {
    if(length(i)>1)
    {
      vmessage(`warning (%s): The following ephemerides match the name "%s":`, _function_name(), ephem_name);
      print_ephem_data(ephem_data[list_to_array(i, Integer_Type)]);
      message("");
    }
    i = i[0];  % pick the first one
    if(ephem_data[i].name != ephem_name)
      vmessage(`warning (%s): using ephemeris "%s" instead of "%s"`, _function_name(), ephem_data[i].name, ephem_name);
    return ephem_data[i];
  }
  else
  {
    vmessage(`error (%s): ephemeris "%s" is not among list of known ephemerides:`, _function_name(), ephem_name);
    print_ephem_data(ephem_data);
  }
}
