define parseDateString()
%!%+
%\function{parseDateString}                                                                                         
%\synopsis{parses a string including year, month, day [, hour [, minute [, second]]]}
%\usage{Struct_Type parseDateString(String_Type date_string);}
%\qualifiers{
%\qualifier{no_2digit_correction}{prevents the correction of two digit year specification,
%                              e.g., 83/12/13 instead of 1983/12/13. If this qualifier
%                              is not set, 2000 is added to years from 00-19 and
%                              1900 to years 20-99.}
%}
%\description
%    This function tries to interprete the given string as a data and returns
%    a structure consistent with the return value of `localtime'. It is
%    assumed that the first number occuring in the string specifies the year,
%    the second the month, and the third the day. If further numbers are found,
%    they are interpreted as hour, minute, second and fraction of seconds. Note
%    that the seconds contained in the structure as defined in `localtime' has
%    to be an integer, while here a double is used in case of fraction of
%    seconds. There can be any separators between the numbers (except numbers).
%\example
%    parseDateString("69/07/21");
%    parseDateString("1969-07-21 02:56:30.44");
%    parseDateString("sdf1969--X07..21QQ_02/56((30,,44xyz");
%\seealso{localtime}
%!%-
{
  variable str;
  switch(_NARGS)
  { case 1: str = (); }
  { help(_function_name()); return;}

  if (length(str)>1)
  { throw RunTimeError, _function_name()+": vector argument cannot be handled"; }
  
  variable matched = 1;
  variable res = {};
  variable len, start, l;
  
  while (matched)
  {
    len = strcharlen (str);
    matched = string_match (str,"\([0-9]+\)"R);
    if (matched)
    {
      (start, l) = string_match_nth(1);
      list_append (res, str[[start:start+l-1]]);
      if ( start+l >= len)
	matched = 0;
      else
	str = str[[start+l:]];
    }
  }

  res = array_map (Integer_Type, &atoi, list_to_array(res));
  variable len_res = length (res);

  % correct two digit years, e.g., 83/12/13 for 1983-12-13 (often used in old radio data)
  ifnot (qualifier_exists("no_2digit_correction"))
    if (len_res >=1 && res[0] < 100) res[0] += (res[0]<20 ? 2000: 1900);
  
  if (len_res < 3)
  { throw RunTimeError, _function_name()+": no proper date found (less than three numbers)"; }
  
  variable i;
  variable d = Integer_Type[7];
  _for i (0,nint(_min(len_res-1,6)),1)
  {
    d[i] = res[i];
  }
  variable sec = d[5];
  if ( d[6] > 0 ) sec = d[5] + double(d[6])/10^(ceil(log10(d[6])));

  % calculate day of year
  variable yday = 0;
  _for i (1, d[1]-1, 1) { yday += daysPerMonth(i, d[0]); }
  yday += d[2];

  % calculate day of week
  variable wday = dayOfWeek(d[0], d[1], d[2]);
  
  return struct {
    tm_sec = sec, tm_min = d[4], tm_hour = d[3],
    tm_mday = d[2], tm_mon = d[1] - 1, tm_year = d[0] - 1900,
    tm_wday = wday mod 7, tm_yday = yday, tm_isdst = -1
  };
}
