define tt2tdb()
%!%+
%\function{tt2tdb}
%\synopsis{Convert a (M)JD in Terrestrial Time (TT, aka TDT or ET) into Terrestrial Barycentric Time (TDB)}
%\usage{tdb=tt2tdb(JD)}
%\description
%This function converts a Julian Date that is given in Terrestrial
%Time into Terrestrial Dynamic Barycentric Time.
%
%By default the routine uses equation 2.6 of Kaplan (2005, USNO Circular 179), which
%is a truncated version of the series provided by Fairhead and Bretagnon,
%1990, A&A 229, 240 and which is better than +/-10mus between AD1600 and AD2200.
%
%If the qualifier ephemeris is given, and points to a JPL ephemeris that
%does contain TT-TDB information, then the value determined from the
%ephemeris is computed (which has a higher precision than Fairhead & Bretagnon,
%but is slower to evaluate).
%
%Note that for virtually all practical applications apart from barycentering
%ms radio pulsar data it is sufficient to use TT as the argument to DE405
%or VSOP1987.
%
%This routine is array safe.
%\qualifiers{
%\qualifier{mjd}{The argument is given in MJD, not in JD.}
%\qualifier{deltat}{Return the difference TDB-TT in seconds, rather than the corrected (M)JD.}
%\qualifier{ephemeris}{if set and no argument, then correct using DE430, otherwise
%                        use the ephemeris provided (see function jpl_initeph).}
%}
%\seealso{tdb2tt,utc2tai,tai2utc,tai2tt,jpl_initeph}
%!%-
{
    variable JD;
    switch(_NARGS)
    {
	case 1: JD=();
    }
    {
	help(_function_name()); return;
    }

    variable dif;
    if (qualifier_exists("ephemeris")) {

	variable ephstruct;
	if (typeof(qualifier("ephemeris"))==Struct_Type) {
	    ephstruct=struct {ephemeris=qualifier("ephemeris")}; % pass to jpl_eph
	} else {
	    ephstruct=struct {dummynameinvalid=0}; % cheat...
	}
	
	variable jdtmp=@JD;
	if (qualifier_exists("mjd")) {
	    jdtmp=jd2mjd(jdtmp);
	}
	
	dif=Double_Type[length(jdtmp)];

	variable i;
	_for i(0,length(jdtmp)-1,1) {
	    variable ddd=(jpl_eph(jdtmp[i],"TT-TDB";;ephstruct))[0]; % ignore time derivative

	    % now iterate until ns precision is reached
	    % (typically only 2 iterations are needed)
	    do {
		variable dold=ddd;
		ddd=(jpl_eph(jdtmp[i]-ddd/86400.,"TT-TDB";;ephstruct))[0]; % ignore time derivative
	    } while (abs(dold-ddd) > 1e-9);
	    dif[i]=-ddd; % TT-TDB -> TDB-TT
	}
	if (typeof(jdtmp)!=Array_Type) {
	    dif=dif[0];
	}
    } else {
	% Fairhead and Bretagnon series
	variable T;
	if (qualifier_exists("mjd")) {
	    T=(JD-51544.5)/36525.;
	} else {
	    T=(JD-2451545.0)/36525.;
	}	
	
        dif=+0.001657*sin( 628.3076*T+6.2401)
            +0.000022*sin( 575.3385*T+4.2970)
            +0.000014*sin(1256.6152*T+6.1969)
            +0.000005*sin( 606.9777*T+4.0212)
            +0.000005*sin(  52.9691*T+0.4444)
            +0.000002*sin(  21.3299*T+5.5431)
            +0.000010*T*sin(628.3076*T+4.2490);
    }
    if (qualifier_exists("deltat")) {
	return dif;
    }
    return JD+dif/86400.0;
}

define tdb2tt()
%!%+
%\function{tdb2tt}
%\synopsis{Convert a (M)JD in Terrestrial Barycentric Time (TDB) into Terrestrial Time (TT, aka TDT)}
%\usage{tt=tdb2tt(JD)}
%\description
%This function converts a Julian Date that is given in Terrestrial Barycentric
%time (TDB) into Terrestrial Time (TT).
%
%By default the routine uses equation 2.6 of Kaplan (2005, USNO Circular 179),
%which is a truncated version of the series provided by Fairhead and Bretagnon,
%1990, A&A 229, 240 and which is better than +/-10mus between AD1600 and
%AD2200.
%
%Since that series is for TT-TDB, we formally calculate the deltat for the TT
%that equals the given JD, we make no attempt to correct for the slight
%change in deltat between JD(TT) and JD(TDB), since the correction is smaller
%than the precision of the Fairhead and Bretagnon series. Use the JPL ephemeris
%for ms pulsar work.
%
%If the qualifier ephemeris is given, and points to a JPL ephemeris that
%does contain TT-TDB information, then the value determined from the
%ephemeris is computed (which has a higher precision than Fairhead & Bretagnon,
%but is slower to evaluate).
%
%Note that for virtually all practical applications apart from barycentering
%ms radio pulsar data it is sufficient to use TT as the argument to DE405
%or VSOP1987.
%
%This routine is array safe.
%\qualifiers{
%\qualifier{mjd}{The argument is given in MJD, not in JD.}
%\qualifier{deltat}{Return the difference TT-TDB in seconds, rather than the corrected (M)JD.}
%\qualifier{ephemeris}{if set and no argument, then correct using DE430, otherwise
%                        use the ephemeris provided (see function jpl_initeph).}
%}
%\seealso{tdb2tt,utc2tai,tai2utc,tai2tt,jpl_initeph}
%!%-
{
    variable JD;
    switch(_NARGS)
    {
	case 1: JD=();
    }
    {
	help(_function_name()); return;
    }

    variable dif;
    if (qualifier_exists("ephemeris")) {

	variable ephstruct;
	if (typeof(qualifier("ephemeris"))==Struct_Type) {
	    ephstruct=struct {ephemeris=qualifier("ephemeris")}; % pass to jpl_eph
	} else {
	    ephstruct=struct {dummynameinvalid=0}; % cheat...
	}
	
	variable jdtmp=@JD;
	if (qualifier_exists("mjd")) {
	    jdtmp=jd2mjd(jdtmp);
	}
	
	dif=Double_Type[length(jdtmp)];

	variable i;
	_for i(0,length(jdtmp)-1,1) {
	    dif[i]=(jpl_eph(jdtmp[i],"TT-TDB";;ephstruct))[0]; % ignore time derivative
	}
	if (typeof(jdtmp)!=Array_Type) {
	    dif=dif[0];
	}
    } else {
	% Fairhead and Bretagnon series for TDB-TT
	variable T;
	if (qualifier_exists("mjd")) {
	    T=(JD-51544.5)/36525.;
	} else {
	    T=(JD-2451545.0)/36525.;
	}	
	
        dif=+0.001657*sin( 628.3076*T+6.2401)
            +0.000022*sin( 575.3385*T+4.2970)
            +0.000014*sin(1256.6152*T+6.1969)
            +0.000005*sin( 606.9777*T+4.0212)
            +0.000005*sin(  52.9691*T+0.4444)
            +0.000002*sin(  21.3299*T+5.5431)
	    +0.000010*T*sin(628.3076*T+4.2490);
	dif=-dif;
    }
    if (qualifier_exists("deltat")) {
	return dif;
    }
    return JD+dif/86400.0;
}
