% -*- mode: slang; mode: fold -*- %

% Gauss Kronrod G7K15 nodes and weights
private variable GK = 
{
  % Nodes
  [0.000000000000000000000000000000000e+00, 2.077849550078984676006894037732449e-01, 4.058451513773971669066064120769615e-01, 5.860872354676911302941448382587296e-01,
   7.415311855993944398638647732807884e-01, 8.648644233597690727897127886409262e-01, 9.491079123427585245261896840478513e-01, 9.914553711208126392068546975263285e-01],
  % Kronrod weights
  [2.094821410847278280129991748917143e-01, 2.044329400752988924141619992346491e-01, 1.903505780647854099132564024210137e-01, 1.690047266392679028265834265985503e-01,
   1.406532597155259187451895905102379e-01, 1.047900103222501838398763225415180e-01, 6.309209262997855329070066318920429e-02, 2.293532201052922496373200805896959e-02],
  % Gauss weights
  [4.179591836734693877551020408163265e-01, 3.818300505051189449503697754889751e-01, 2.797053914892766679014677714237796e-01, 1.294849661688696932706114326790820e-01]
};

define integrateGK ()
%!%+
%\function{integrateGK}
%\synopsis{Integrate function with Gauss Kronrod method (G7K15)}
%\usage{Double_Type integrateGK(Ref_Typ fun, Double_Type a, b, ...);}
%\qualifiers{
%  \qualifier{max_intervals}{[=5]: Maximum number of intervals to use}
%}
%\description
%  Calculate the integral of a function \code{fun} from \code{a}
%  to \code{b}. It is expected that the first argument of \code{fun}
%  is the integration parameter. Additional arguments can be passed
%  after the integration boundaries.
%
%\example
%  variable v = integrateGK(&sin, 0, PI);
%\seealso{integrateRK4}
%!%-
{
  variable args = __pop_list(_NARGS-3);
  variable fun, a, b;
  (fun, a, b) = ();
  variable maxIntervals = qualifier("max_intervals", 5);
  variable leftBorder = Double_Type[maxIntervals];
  variable rightBorder = Double_Type[maxIntervals];
  variable errorValue = Double_Type[maxIntervals];
  variable integralValue = Double_Type[maxIntervals];
  variable integralCalculated = Int_Type[maxIntervals];
  variable totalIntegral, totalError;
  variable currentI = 0;
  variable newI = 1;
  variable lengthI = 1;
  variable max_inter = 0;
  variable m,d,x,y,intG,intK;

  leftBorder[0] = a;
  rightBorder[0] = b;
  integralCalculated[0] = 0;

  if (abs(a-b)<1e-12)
    return 1e-12*@fun(0.5*(b-a), __push_list(args));

  while (lengthI<maxIntervals)
  {
    ifnot (integralCalculated[currentI])
    {
      a = leftBorder[currentI];
      b = rightBorder[currentI];
      m = 0.5*(a+b);
      d = 0.5*(b-a);
      x = [GK[0][[::2]]*d+m, -GK[0][[2::2]]*d+m];
      y = @fun(x, __push_list(args);; __qualifiers());

      % Gauss nodes
      intK = sum(y*[GK[1][[::2]], GK[1][[2::2]]]);
      intG = sum(y*[GK[2], GK[2][[1:]]]);

      % Kronrod nodes
      x = [GK[0][[1::2]]*d+m, -GK[0][[1::2]]*d+m];
      y = @fun(x, __push_list(args);; __qualifiers());
      intK += sum(y*[GK[1][[1::2]],GK[1][[1::2]]]);

      integralValue[currentI] = intK*d;
      errorValue[currentI] = abs(intG-intK)*d;
    }

    if (currentI < newI)
    {
      totalIntegral = sum(integralValue[[currentI:newI-1]]);
      totalError = sumsq(errorValue[[currentI:newI-1]]);
    }
    else
    {
      totalIntegral = sum(integralValue[[[currentI:maxIntervals-1], [0:newI-1]]]);
      totalError = sumsq(errorValue[[[currentI:maxIntervals-1], [0:newI-1]]]);
    }

    if (max_inter<lengthI) { max_inter = lengthI; }
    if (abs(sqrt(totalError)/totalIntegral)<1e-8
	|| (abs(totalIntegral)<1e-14 && abs(totalError)<1e-16))
      return totalIntegral;

    if ( (not integralCalculated[currentI]) &&
	 ((abs(errorValue[currentI]/integralValue[currentI])>1e-8)
	  || (abs(integralValue[currentI])<1e-14 && abs(errorValue[currentI])<1e-16)))
    {
      m = 0.5*(leftBorder[currentI]+rightBorder[currentI]);
      leftBorder[newI] = leftBorder[currentI];
      rightBorder[newI] = m;
      errorValue[newI] = _Inf;

      newI++;
      if (newI==maxIntervals) newI = 0;

      leftBorder[newI] = m;
      rightBorder[newI] = rightBorder[currentI];
      errorValue[newI] = _Inf;

      newI++;
      if (newI==maxIntervals) newI = 0;

      lengthI++;
    }
    else
    {
      integralValue[newI] = integralValue[currentI];
      errorValue[newI] = errorValue[currentI];
      integralCalculated[newI] = 1;
      integralCalculated[currentI] = 0;
      newI++;

      if (newI==maxIntervals)
	newI = 0;
    }

    currentI++;
    if (currentI == maxIntervals)
      currentI = 0;
  }

  return totalIntegral;
}
